/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		29.01.2009
LastModified:	...

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
-----------------------------------------------------------------------*/
#ifndef _2D_COLLISION_H_INCLUDED_
#define _2D_COLLISION_H_INCLUDED_

#include "DEFINE_FOR_BORLANDC"

#include "IDPEObject.h"

namespace DPE_Scenes
{

	namespace _2D
	{
		// forward
		interface IIterator;
		interface IObject;
		interface IBody;

		//-----------------------------------------------------------------------
		//ICollisionShape//
		//-----------------------------------------------------------------------

		/// Collision shape.
		/** Do not try to query this interface. */
		interface ICollisionShape
		{
			/// Shape type enumaration.
			enum EShapeType
			{
				e_unknownShape = -1,	///< an unknown shape for base class. Do not use this.
				e_circleShape,			///< a circle shape.
				e_polygonShape			///< a polygon shape.
			};

			#pragma pack(push, 1)

			/// Contact filtering descriptor.
			struct TFilterDesc
			{
				/// The collision category bits. Normally you would just set one bit.
				unsigned short categoryBits;

				/// The collision mask bits. This states the categories that this
				/// shape would accept for collision.
				unsigned short maskBits;

				/// Collision groups allow a certain group of objects to never collide (negative)
				/// or always collide (positive). Zero means no collision group. Non-zero group
				/// filtering always wins against the mask bits.
				signed short groupIndex;
			};

			/// Base shape descriptor.
			struct TShapeDesc
			{
				/// Default constructor for shape descriptor.
				inline TShapeDesc()
				{
					type = e_unknownShape;
					friction = 0.2f;
					restitution = 0.0f;
					density = 0.0f;
					filter.categoryBits = 0x0001;
					filter.maskBits = 0xFFFF;
					filter.groupIndex = 0;
					isSensor = false;
				}

				/// Shape type for down-casting.
				EShapeType type;

				/// The shape's friction coefficient, usually in the range [0,1].
				float friction;

				/// The shape's restitution (elasticity) usually in the range [0,1].
				float restitution;

				/// The shape's density, usually in kg/m^2.
				float density;

				/// A sensor shape collects contact information but never generates a collision
				/// response.
				bool isSensor;

				/// Contact filtering desc.
				TFilterDesc filter;
			};
			
			/// Circle shape descriptor.
			struct TCircleDesc : public TShapeDesc
			{
				/// Default costructor.
				inline TCircleDesc() : TShapeDesc()
				{
					type = e_circleShape;			
					radius = 1.0f;
				}
				/// Center of circle shape in local coordinates.
				Point2D_Int localPosition;

				/// Circle shape radius.
				float radius;
			};

			/// Convex polygon.
			/** The vertices must be in CW order. */
			struct TPolygonDesc : public TShapeDesc
			{
				/// Default constructor
				inline TPolygonDesc() : TShapeDesc()
				{
					type = e_polygonShape;
					vertexCount = 0;
				}
				/// Build vertices to represent an axis-aligned box.
				/// @param width is the half-width.
				/// @param height is the half-height.
				inline void SetAsBox(int width, int height)
				{
					vertexCount = 4;
					vertexCount = 4;
					vertices[0] = Point2D_Int(-width,-height);
					vertices[1] = Point2D_Int(width,-height);
					vertices[2] = Point2D_Int(width,height);
					vertices[3] = Point2D_Int(-width,height);
				}

				/// The polygon vertices in local coordinates.
				Point2D_Int vertices[8];

				/// The number of polygon vertices.
				int vertexCount;
			};
			#pragma pack(pop)

			/// Copy shape descriptor and assign to object.
			/// @param[in] pShapeDesc the shape descriptor.
			virtual HRESULT CALLBACK SetShape(ICollisionShape::TShapeDesc *pShapeDesc) = 0;

			/// Returns shape descriptor of object.
			/// @param[out] pShapeDesc the shape descriptor.
			virtual HRESULT CALLBACK GetShape(OUT ICollisionShape::TShapeDesc *&pShapeDesc) = 0;

			/// Reset shape descriptor of object. Set object not collidable.
			virtual HRESULT CALLBACK UnsetShape( void ) = 0;

			/// Setups collidable body.
			/// @note Doesn't increment reference. Use NULL to unset.
			/// @warning for internal purposes.
			/// @param pBody the body.
			virtual HRESULT CALLBACK SetBody(IBody *pBody) = 0;

			/// Returns collidable body.
			/// @note Increment reference.
			/// @warning for internal purposes.
			/// @param[out] pBody the body.
			virtual HRESULT CALLBACK GetBody(OUT IBody *&pBody) const = 0;
		};

		//-----------------------------------------------------------------------
		//IBody//
		//-----------------------------------------------------------------------
		// {A1433FFC-D917-4d04-84B6-6DF7EC1BADA6}
		static const GUID IID_IBody = 
		{ 0xa1433ffc, 0xd917, 0x4d04, { 0x84, 0xb6, 0x6d, 0xf7, 0xec, 0x1b, 0xad, 0xa6 } };

		/// Default ground static body.
		/** Without shapes, mass and position in (0,0). */
		#define DEFAULT_GROUND_BODY "ground_body"

		/// Collidable body.
		/**	Base unit for collision test. Includes shapes of objects.
			To destroy body you should destroy first all joints that use it,
			call free (@see IBody::Free()) and release all references in your program.
			If your destroy collidable body, all shapes (@see IObject) will remain.
			If they not needed than destroy them manually.
		*/
		/// @see ICollisionShape.
		/// @see IObject.
		interface __declspec(uuid("{A1433FFC-D917-4d04-84B6-6DF7EC1BADA6}")) IBody: public IDPEObject, public IFree
		{
			#pragma pack(push, 1)
			
			/// Mass descriptor for shape.
			/** Use this to set mass of body manually.
				The recommended way of specifying body mass is by setting the shape densities
				and calling IBody::SetMassFromShapes().
				@see IBody::SetMass()
				@see IBody::SetMassFromShapes()
			*/
			struct TMassDesc
			{
				/// The mass of the shape, usually in kilograms.
				float mass;

				/// The position of the shape's centroid relative to the shape's origin.
				Point2D_Int center;

				/// The rotational inertia of the shape.
				float I;
			};

			struct TBodyDesc
			{
				TBodyDesc()
				{
					massData.mass = 0.0f;
					massData.I = 0.0f;
					angle = 0.0f;
					linearDamping = 0.0f;
					angularDamping = 0.0f;
					allowSleep = true;
					isSleeping = false;
					fixedRotation = false;
					isBullet = false;
				}

				/// You can use this to initialized the mass properties of the body.
				/// If you prefer, you can set the mass properties after the shapes
				/// have been added using b2Body::SetMassFromShapes.
				IBody::TMassDesc massData;

				/// The world position of the body. Avoid creating bodies at the origin
				/// since this can lead to many overlapping shapes.
				Point2D_Int position;

				/// The world angle of the body in radians.
				float angle;

				/// Linear damping is use to reduce the linear velocity. The damping parameter
				/// can be larger than 1.0f but the damping effect becomes sensitive to the
				/// time step when the damping parameter is large.
				float linearDamping;

				/// Angular damping is use to reduce the angular velocity. The damping parameter
				/// can be larger than 1.0f but the damping effect becomes sensitive to the
				/// time step when the damping parameter is large.
				float angularDamping;

				/// Set this flag to false if this body should never fall asleep. Note that
				/// this increases CPU usage.
				bool allowSleep;

				/// Is this body initially sleeping?
				bool isSleeping;

				/// Should this body be prevented from rotating? Useful for characters.
				bool fixedRotation;

				/// Is this a fast moving body that should be prevented from tunneling through
				/// other moving bodies? Note that all bodies are prevented from tunneling through
				/// static bodies.
				/// @warning You should use this flag sparingly since it increases processing time.
				bool isBullet;
			};
			#pragma pack(pop)

			/// Append shape of the object.
			/// @param[in] pObject the object shapes to add to body.
			virtual HRESULT CALLBACK AppendShapes(IN IObject *pObject) = 0;
			/// Delete shape of the object.
			/// @param pObject the object shape to remove from body.
			virtual HRESULT CALLBACK DeleteShape(IObject *pObject) = 0;

			/// Set the mass properties. Note that this changes the center of mass position.
			/// If you are not sure how to compute mass properties, use SetMassFromShapes.
			/// The inertia tensor is assumed to be relative to the center of mass.
			/// @param[in] pMassDesc the mass properties.
			virtual HRESULT CALLBACK SetMass(const IBody::TMassDesc *pMassDesc) = 0;

			/// Compute the mass properties from the attached shapes. You typically call this
			/// after adding all the shapes. If you add or remove shapes later, you may want
			/// to call this again. Note that this changes the center of mass position.
			virtual HRESULT CALLBACK SetMassFromShapes() = 0;

			/// Set the position of the body's origin.
			/// This breaks any contacts and wakes the other bodies.
			/// If the movement put a shape outside the world. In this case the
			/// body is automatically frozen.
			/// @param pos the new world position of the body's origin.
			virtual HRESULT CALLBACK SetPosition(const Point2D_Int &pos) = 0;

			/// Set rotation in radians.
			/// This breaks any contacts and wakes the other bodies.
			/// If the movement put a shape outside the world. In this case the
			/// body is automatically frozen.
			/// @param angle the new world rotation angle of the body in radians.
			virtual HRESULT CALLBACK SetAngle(float angle) = 0;

			/// Returns the world body origin position.
			// @param[out] pos the world body origin position in px.
			virtual HRESULT CALLBACK GetPosition(OUT Point2D_Int &pos) const = 0;

			/// Returns the angle in radians.
			/// @param[out] angle the angle in radians.
			virtual HRESULT CALLBACK GetAngle(OUT float &angle) const = 0;

			/// Returns the world position of the center of mass.
			/// @param[out] pos the world position in px.
			virtual HRESULT CALLBACK GetWorldCenter(OUT Point2D_Int &pos) const = 0;

			/// Returns the local position of the center of mass.
			/// @param[out] pos the local position in px.
			virtual HRESULT CALLBACK GetLocalCenter(OUT Point2D_Int &pos) const = 0;

			/// Setups the linear velocity of the center of mass.
			/// @param v the new linear velocity in px per second.
			virtual HRESULT CALLBACK SetLinearVelocity(const Vector2D_Float& v) = 0;

			/// Returns the linear velocity of the center of mass.
			/// @param[out] v the velocity in px per second.
			virtual HRESULT CALLBACK GetLinearVelocity(OUT Vector2D_Float &v) const = 0;

			/// Setups the angular velocity.
			/// @param omega the new angular velocity in radians/second.
			virtual HRESULT CALLBACK SetAngularVelocity(float omega) = 0;

			/// Returns the angular velocity.
			/// @param omega the angular velocity in radians/second.
			virtual HRESULT CALLBACK GetAngularVelocity(OUT float &omega) const = 0;

			/// Apply a force at a world point. If the force is not
			/// applied at the center of mass, it will generate a torque and
			/// affect the angular velocity. This wakes up the body.
			/// @param vForce the world force vector, usually in Newtons (N).
			/// @param point the world position of the point of application.
			virtual HRESULT CALLBACK ApplyForce(const Vector2D_Float &vForce, const Point2D_Int &point) = 0;

			/// Apply a torque. This affects the angular velocity
			/// without affecting the linear velocity of the center of mass.
			/// This wakes up the body.
			/// @param torque about the z-axis (out of the screen), usually in N-m.
			virtual HRESULT CALLBACK ApplyTorque(float torque) = 0;

			/// Apply an impulse at a point. This immediately modifies the velocity.
			/// It also modifies the angular velocity if the point of application
			/// is not at the center of mass. This wakes up the body.
			/// @param vImpulse the world impulse vector, usually in N-seconds or kg-m/s.
			/// @param point the world position of the point of application.
			virtual HRESULT CALLBACK ApplyImpulse(const Vector2D_Float &vImpulse, const Point2D_Int &point) = 0;

			/// Get the total mass of the body, usually in kilograms (kg).
			virtual HRESULT CALLBACK GetMass(OUT float &mass) const = 0;

			/// Get the central rotational inertia of the body, usually in kg-m^2.
			virtual HRESULT CALLBACK GetInertia(OUT float &inertia) const = 0;

			/// Returns the world coordinates of a point given in the local coordinates.
			/// @param localPoint a point on the body measured relative the the body's origin.
			/// @param[out] worldPoint the same point expressed in world coordinates in px.
			virtual HRESULT CALLBACK GetWorldPoint(const Point2D_Int &localPoint, OUT Point2D_Int &worldPoint) const = 0;

			/// Returns the world coordinates of a vector given in the local coordinates.
			/// @param localVector a vector fixed in the body.
			/// @param[out] worldVector the same vector expressed in world coordinates in px.
			virtual HRESULT CALLBACK GetWorldVector(const Vector2D_Float &localVector, OUT Vector2D_Float &worldVector) const = 0;

			/// Returns a local point relative to the body's origin given a world point.
			/// @param worldPoint a point in world coordinates.
			/// @param[out] localPoint the corresponding local point relative to the body's origin in px.
			virtual HRESULT CALLBACK GetLocalPoint(const Point2D_Int &worldPoint, OUT Point2D_Int &localPoint) const = 0;

			/// Returns a local vector given a world vector.
			/// @param worldVector a vector in world coordinates.
			/// @param[out] localVector the corresponding local vector in px.
			virtual HRESULT CALLBACK GetLocalVector(const Vector2D_Float &worldVector, OUT Vector2D_Float &localPoint) const = 0;

			/// Get the world linear velocity of a world point attached to this body.
			/// @param worldPoint the point in world coordinates.
			/// @param[out] vVelocity the world velocity of a point in px.
			virtual HRESULT CALLBACK GetLinearVelocityFromWorldPoint(const Point2D_Int &worldPoint, OUT Vector2D_Float &vVelocity) const = 0;

			/// Get the world velocity of a local point.
			/// @param localPoint the point in local coordinates.
			/// @param[out] vVelocity the world velocity of a point in px.
			virtual HRESULT CALLBACK GetLinearVelocityFromLocalPoint(const Point2D_Int &localPoint, OUT Vector2D_Float &vVelocity) const = 0;

			/// Is this body treated like a bullet for continuous collision detection?
			virtual HRESULT CALLBACK IsBullet(OUT bool &bBullet)const  = 0;

			/// Should this body be treated like a bullet for continuous collision detection?
			virtual HRESULT CALLBACK SetBullet(bool bFlag) const = 0;

			/// Is this body static (immovable)?
			virtual HRESULT CALLBACK IsStatic(OUT bool &bStatic) const = 0;

			/// Is this body dynamic (movable)?
			virtual HRESULT CALLBACK IsDynamic(OUT bool &bDynamic) const = 0;

			/// Is this body frozen?
			virtual HRESULT CALLBACK IsFrozen(OUT bool &bFrozen) const = 0;

			/// Is this body sleeping (not simulating).
			virtual HRESULT CALLBACK IsSleeping(OUT bool &bSleeping) const = 0;

			/// You can disable sleeping on this body.
			virtual HRESULT CALLBACK AllowSleeping(bool bFlag) = 0;

			/// Wake up this body so it will begin simulating.
			virtual HRESULT CALLBACK WakeUp( void ) = 0;

			/// Put this body to sleep so it will stop simulating.
			/// This also sets the velocity to zero.
			virtual HRESULT CALLBACK PutToSleep( void ) = 0;
		};

		//-----------------------------------------------------------------------
		//IJoint//
		//-----------------------------------------------------------------------

		// {84D1EFE7-D989-48bc-B224-876555799FE1}
		static const GUID IID_IJoint = 
		{ 0x84d1efe7, 0xd989, 0x48bc, { 0xb2, 0x24, 0x87, 0x65, 0x55, 0x79, 0x9f, 0xe1 } };

		/// A base interface of joint.
		/** To destroy joint just call free (@see IBody::Free()) 
			and release all references in your program.
		*/
		interface __declspec(uuid("{84D1EFE7-D989-48bc-B224-876555799FE1}")) IJoint: public IDPEObject, public IFree
		{
			/// A joint type enumaration.
			enum EJointType
			{
				e_unknownJoint,				///< an unknown type.
				e_revoluteJoint,			///< a revolute type.
				e_prismaticJoint,			///< a prismatic type.
				e_distanceJoint,			///< a distance type.
				e_pulleyJoint,				///< a pulley type.
				e_mouseJoint,				///< a mouse type.
				e_gearJoint					///< a gear type.
			};

			#pragma pack(push, 1)

			/// Joint descriptor.
			/** Is used to construct joints. */
			struct TJointDesc
			{
				/// Default constructor.
				TJointDesc()
				{
					type = e_unknownJoint;
					pBody1 = NULL;
					pBody2 = NULL;
					collideConnected = false;
				}

				/// The joint type is set automatically for concrete joint types.
				EJointType type;

				/// The first attached body.
				/// @warning After creating body reference will be incremented.
				IBody *pBody1;

				/// The second attached body.
				/// @warning After creating body reference will be incremented.
				IBody *pBody2;

				/// Set this flag to true if the attached bodies should collide.
				bool collideConnected;
			};
			#pragma pack(pop)


			/// Returns the first body attached to this joint.
			/// @note Increments body reference.
			/// @param[out] pBody the first attached body.
			virtual HRESULT CALLBACK GetBody1(OUT IBody *&pBody) const = 0;

			/// Returns the second body attached to this joint.
			/// @note Increments body reference.
			/// @param[out] pBody the second attached body.
			virtual HRESULT CALLBACK GetBody2(OUT IBody *&pBody) const = 0;

			/// Returns the anchor point on body1 in world coordinates.
			/// @param[out] anchor the anchor point.
			virtual HRESULT CALLBACK GetAnchor1(OUT Point2D_Int &anchor) const = 0;

			/// Returns the anchor point on body2 in world coordinates.
			/// @param[out] anchor the anchor point.
			virtual HRESULT CALLBACK GetAnchor2(OUT Point2D_Int &anchor) const = 0;

			/// Get the reaction force on body2 at the joint anchor.
			/// @param[out] force the reaction force on body2.
			virtual HRESULT CALLBACK GetReactionForce(OUT Vector2D_Float &force) const = 0;

			/// Get the reaction torque on body2.
			/// @param[out] force the reaction torque on body2.
			virtual HRESULT CALLBACK GetReactionTorque(OUT float &torque) const = 0;
		};

		//-----------------------------------------------------------------------
		//IDistanceJoint//
		//-----------------------------------------------------------------------

		// {5DCE3784-CE89-43ea-98A6-E2A86C8090CD}
		static const GUID IID_IDistanceJoint = 
		{ 0x5dce3784, 0xce89, 0x43ea, { 0x98, 0xa6, 0xe2, 0xa8, 0x6c, 0x80, 0x90, 0xcd } };

		/// A distance joint.
		/** Constrains two points on two bodies
			to remain at a fixed distance from each other.
			You can view this as a massless, rigid rod.
		*/
		interface __declspec(uuid("{5DCE3784-CE89-43ea-98A6-E2A86C8090CD}")) IDistanceJoint: public IJoint
		{
			#pragma pack(push, 1)

			/// Distance joint descriptor.
			/** This requires defining an anchor point on both bodies
				and the non-zero length of the distance joint.
				The definition uses local anchor points so that the initial configuration
				can violate the constraint slightly. This helps when saving and loading a game.
			*/
			/// @warning Do not use a zero or short length.
			struct TDistanceJointDesc : public IJoint::TJointDesc
			{
				/// Default constructor
				TDistanceJointDesc() : IJoint::TJointDesc()
				{
					type = IJoint::e_distanceJoint;
					length = 10.0f;
					frequencyHz = 0.0f;
					dampingRatio = 0.0f;
				}

				/// Initialize the bodies, anchors, and length using the world
				/// anchors.
				inline void Initialize(IBody *pBody1, IBody *pBody2,
					const Point2D_Int &anchor1, const Point2D_Int &anchor2)
				{
					IJoint::TJointDesc::pBody1 = pBody1;
					IJoint::TJointDesc::pBody2 = pBody2;
					pBody1->GetLocalPoint(anchor1, localAnchor1);
					pBody2->GetLocalPoint(anchor2, localAnchor2);
					length = sqrtf(anchor2.distanceSqr(anchor1));
				}

				/// The local anchor point relative to body1's origin.
				Point2D_Int localAnchor1;

				/// The local anchor point relative to body2's origin.
				Point2D_Int localAnchor2;

				/// The equilibrium length between the anchor points.
				float length;

				/// The response speed.
				float frequencyHz;

				/// The damping ratio. 0 = no damping, 1 = critical damping.
				float dampingRatio;
			};
			#pragma pack(pop)

			/// Setups length between the anchor points.
			/// @param length the new length in px.
			virtual HRESULT CALLBACK SetLength(float length) = 0;

			/// Returns length between the anchor points.
			/// @param[out] length the length in px.
			virtual HRESULT CALLBACK GetLength(OUT float &length) = 0;
		};

		//-----------------------------------------------------------------------
		//IRevoluteJoint//
		//-----------------------------------------------------------------------

		// {49E5B51B-A35E-4f7e-A58E-29486A8E6DBE}
		static const GUID IID_IRevoluteJoint = 
		{ 0x49e5b51b, 0xa35e, 0x4f7e, { 0xa5, 0x8e, 0x29, 0x48, 0x6a, 0x8e, 0x6d, 0xbe } };

		/// A revolute joint.
		/** A revolute joint constrains to bodies to share a common point while they
			are free to rotate about the point. The relative rotation about the shared
			point is the joint angle. You can limit the relative rotation with
			a joint limit that specifies a lower and upper angle. You can use a motor
			to drive the relative rotation about the shared point. A maximum motor torque
			is provided so that infinite forces are not generated.
		*/
		interface __declspec(uuid("{5DCE3784-CE89-43ea-98A6-E2A86C8090CD}")) IRevoluteJoint: public IJoint
		{
			/// Revolute joint descriptor.
			/** This requires defining an anchor point where the bodies are joined.
				The descriptor uses local anchor points so that the initial configuration
				can violate the constraint slightly. You also need 
				to specify the initial relative angle for joint limits.
			*/
			#pragma pack(push, 1)
			struct TRevoluteJointDesc : public IJoint::TJointDesc
			{
				/// Default TRevoluteJointDesc constructor.
				TRevoluteJointDesc() : IJoint::TJointDesc()
				{
					type = IJoint::e_revoluteJoint;
					referenceAngle = 0.0f;
					lowerAngle = 0.0f;
					upperAngle = 0.0f;
					maxMotorTorque = 0.0f;
					motorSpeed = 0.0f;
					enableLimit = false;
					enableMotor = false;
				}

				/// Initialize the bodies, anchors, and reference angle using the world
				/// anchor.
				inline void Initialize(IBody *pBody1, IBody *pBody2,
					const Point2D_Int &anchor)
				{
					IJoint::TJointDesc::pBody1 = pBody1;
					IJoint::TJointDesc::pBody2 = pBody2;
					pBody1->GetLocalPoint(anchor, localAnchor1);
					pBody2->GetLocalPoint(anchor, localAnchor2);
					float angle1, angle2;
					pBody1->GetAngle(angle1);
					pBody2->GetAngle(angle2);
					referenceAngle = angle2 - angle1;
				}

				/// The local anchor point relative to body1's origin.
				Point2D_Int localAnchor1;

				/// The local anchor point relative to body2's origin.
				Point2D_Int localAnchor2;

				/// The body2 angle minus body1 angle in the reference state (radians).
				float referenceAngle;

				/// A flag to enable joint limits.
				bool enableLimit;

				/// The lower angle for the joint limit (radians).
				float lowerAngle;

				/// The upper angle for the joint limit (radians).
				float upperAngle;

				/// A flag to enable the joint motor.
				bool enableMotor;

				/// The desired motor speed. Usually in radians per second.
				float motorSpeed;

				/// The maximum motor torque used to achieve the desired motor speed.
				/// Usually in N-m.
				float maxMotorTorque;
			};
			#pragma pack(pop)

			/// Returns the current joint angle in radians.
			/// @param[out] angle the current joint angle.
			virtual HRESULT CALLBACK GetJointAngle(OUT float &angle) const = 0;

			/// Returns the current joint angle speed in radians per second.
			/// @param[out] speed the angle speed.
			virtual HRESULT CALLBACK GetJointSpeed(OUT float &speed) const = 0;

			/// Is the joint limit enabled?
			/// @param[out] enable flag.
			virtual HRESULT CALLBACK IsLimitEnabled(OUT bool &enable) const = 0;

			/// Enable/disable the joint limit.
			/// @param enable the flag.
			virtual HRESULT CALLBACK EnableLimit(bool enable) = 0;

			/// Returns the lower joint limit in radians.
			/// @param[out] limit in in radians.
			virtual HRESULT CALLBACK GetLowerLimit(OUT float &limit) const = 0;

			/// Returns the upper joint limit in radians.
			/// @param[out] limit in in radians.
			virtual HRESULT CALLBACK GetUpperLimit(OUT float &limit) const = 0;

			/// Setups the joint limits in radians.
			/// @param lower limit in in radians.
			/// @param upper limit in in radians.
			virtual HRESULT CALLBACK SetLimits(float lower, float upper) = 0;

			/// Is the joint motor enabled?
			/// @param[out] enable flag.
			virtual HRESULT CALLBACK IsMotorEnabled(OUT bool &enable) const = 0;

			/// Enable/disable the joint motor.
			/// @param enable flag.
			virtual HRESULT CALLBACK EnableMotor(bool enable) = 0;

			/// Setups the motor speed in radians per second.
			/// @param speed the motor speed in radians per second.
			virtual HRESULT CALLBACK SetMotorSpeed(float speed) = 0;

			/// Returns the motor speed in radians per second.
			/// @param[out] speed the motor in radians per second.
			virtual HRESULT CALLBACK GetMotorSpeed(OUT float &speed) const = 0;

			/// Set the maximum motor torque, usually in N-m.
			/// @param torque the maximum motor torque.
			virtual HRESULT CALLBACK SetMaxMotorTorque(float torque) = 0;

			/// Returns the current motor torque, usually in N-m.
			/// @param[out] torque the current motor torque.
			virtual HRESULT CALLBACK GetMotorTorque(OUT float &torque) const = 0;
		};

		//-----------------------------------------------------------------------
		//IPrismaticJoint//
		//-----------------------------------------------------------------------

		// {96340AAD-9064-4404-A8A4-3BA69594C1ED}
		static const GUID IID_IPrismaticJoint = 
		{ 0x96340aad, 0x9064, 0x4404, { 0xa8, 0xa4, 0x3b, 0xa6, 0x95, 0x94, 0xc1, 0xed } };

		/// A prismatic joint.
		/** This joint provides one degree of freedom: translation along an axis fixed in body1.
			Relative rotation is prevented. You can use a joint limit to restrict the range of motion
			and a joint motor to drive the motion or to model joint friction.
		*/
		interface __declspec(uuid("{96340AAD-9064-4404-A8A4-3BA69594C1ED}")) IPrismaticJoint: public IJoint
		{
			/// Prismatic joint descriptor.
			/** This requires defining a line of motion using an axis and an anchor point.
				The definition uses local anchor points and a local axis
				so that the initial configuration can violate the constraint slightly.
				The joint translation is zero when the local anchor points coincide in world space.
			*/		
			#pragma pack(push, 1)
			struct TPrismaticJointDesc : public IJoint::TJointDesc
			{
				TPrismaticJointDesc() : IJoint::TJointDesc()
				{
					type = e_prismaticJoint;
					referenceAngle = 0.0f;
					enableLimit = false;
					lowerTranslation = 0.0f;
					upperTranslation = 0.0f;
					enableMotor = false;
					maxMotorForce = 0.0f;
					motorSpeed = 0.0f;
				}

				/// Initialize the bodies, anchors, axis, and reference angle using the world
				/// anchor and world axis.
				inline void Initialize(IBody *pBody1, IBody *pBody2,
					const Point2D_Int &anchor, const Vector2D_Float &axis)
				{
					IJoint::TJointDesc::pBody1 = pBody1;
					IJoint::TJointDesc::pBody2 = pBody2;
					pBody1->GetLocalPoint(anchor, localAnchor1);
					pBody2->GetLocalPoint(anchor, localAnchor2);
					pBody1->GetLocalVector(axis, localAxis1);
					float angle1, angle2;
					pBody1->GetAngle(angle1);
					pBody2->GetAngle(angle2);
					referenceAngle = angle2 - angle1;
				}

				/// The local anchor point relative to body1's origin.
				Point2D_Int localAnchor1;

				/// The local anchor point relative to body2's origin.
				Point2D_Int localAnchor2;

				/// The local translation axis in body1.
				Vector2D_Float localAxis1;

				/// The constrained angle between the bodies: body2_angle - body1_angle.
				float referenceAngle;

				/// Enable/disable the joint limit.
				bool enableLimit;

				/// The lower translation limit in px.
				float lowerTranslation;

				/// The upper translation limit in px.
				float upperTranslation;

				/// Enable/disable the joint motor.
				bool enableMotor;

				/// The maximum motor torque, usually in N-m.
				float maxMotorForce;

				/// The desired motor speed in radians per second.
				float motorSpeed;
			};
			#pragma pack(pop)

			/// Returns the current joint translation.
			/// @param[out] length the current joint translation in px.
			virtual HRESULT CALLBACK GetJointTranslation(OUT float &length) const = 0;

			/// Returns the current joint translation speed.
			/// @param[out] speed  the current translation speed in px per second.
			virtual HRESULT CALLBACK GetJointSpeed(OUT float &speed) const = 0;

			/// Is the joint limit enabled?
			/// @param[out] enable the flag.
			virtual HRESULT CALLBACK IsLimitEnabled(OUT bool &enable) const = 0;

			/// Enable/disable the joint limit.
			/// @param enable the flag.
			virtual HRESULT CALLBACK EnableLimit(bool enable) = 0;

			/// Returns the lower joint limit.
			/// @param[out] limit the lower joint limit in px.
			virtual HRESULT CALLBACK GetLowerLimit(OUT float &limit) const = 0;

			/// Returns the upper joint limit.
			/// @param[out] limit the upper joint limit in px.
			virtual HRESULT CALLBACK GetUpperLimit(OUT float &limit) const = 0;

			/// Setups the joint limits.
			/// @param lower the lower joint limit in px.
			/// @param upper the upper joint limit in px.
			virtual HRESULT CALLBACK SetLimits(float lower, float upper) = 0;

			/// Is the joint motor enabled?
			/// @param enable the flag.
			virtual HRESULT CALLBACK IsMotorEnabled(OUT float &enable) const = 0;

			/// Enable/disable the joint motor.
			/// @param enable the flag.
			virtual HRESULT CALLBACK EnableMotor(bool enable) = 0;

			/// Setups the motor speed.
			/// @param speed the motor speed in px per second.
			virtual HRESULT CALLBACK SetMotorSpeed(float speed) = 0;

			/// Returns the motor speed.
			/// @param[out] speed the motor speed in px per second.
			virtual HRESULT CALLBACK GetMotorSpeed(OUT float &speed) const = 0;

			/// Setups the maximum motor force, usually in N.
			/// @param force the maximum motor force.
			virtual HRESULT CALLBACK SetMaxMotorForce(float force) = 0;

			/// Returns the current motor force, usually in N.
			/// @param[out] force the maximum motor force.
			virtual HRESULT CALLBACK GetMotorForce(OUT float &force) const = 0;
		};

		//-----------------------------------------------------------------------
		//IPulleyJoint//
		//-----------------------------------------------------------------------

		// {D293FA3E-6A0A-4202-A4C0-E50E90BAD3B9}
		static const GUID IID_IPulleyJoint = 
		{ 0xd293fa3e, 0x6a0a, 0x4202, { 0xa4, 0xc0, 0xe5, 0xe, 0x90, 0xba, 0xd3, 0xb9 } };

		/// The pulley joint.
		/**	Joint is connected to two bodies and two fixed ground points.
			The pulley supports a ratio such that: length1 + ratio * length2 <= constant.
			Yes, the force transmitted is scaled by the ratio.
			The pulley also enforces a maximum length limit on both sides.
			This is useful to prevent one side of the pulley hitting the top.
		*/
		static const float MIN_LENGTH = 2.0f;

		interface __declspec(uuid("{D293FA3E-6A0A-4202-A4C0-E50E90BAD3B9}")) IPulleyJoint: public IJoint
		{

			/// Pulley joint descriptor.
			/**	This requires two ground anchors, two dynamic body anchor points,
				max lengths for each side, and a pulley ratio.
			*/
			#pragma pack(push, 1)
			struct TPulleyJointDesc : public IJoint::TJointDesc
			{
				TPulleyJointDesc() : IJoint::TJointDesc()
				{
					type = e_pulleyJoint;
					groundAnchor1 = Point2D_Int(-10, 10);
					groundAnchor1 = Point2D_Int(10, 10);
					localAnchor1 = Point2D_Int(-10, 0);
					localAnchor2 = Point2D_Int(10, 0);
					length1 = 0.0f;
					maxLength1 = 0.0f;
					length2 = 0.0f;
					maxLength2 = 0.0f;
					ratio = 1.0f;
					collideConnected = true;
				}

				/// Initialize the bodies, anchors, lengths, max lengths, and ratio using the world anchors.
				inline void Initialize(IBody *pBody1, IBody *pBody2,
					const Point2D_Int &groundAnchor1, const Point2D_Int &groundAnchor2,
					const Point2D_Int &anchor1, const Point2D_Int &anchor2,
					float ratio)
				{
					IJoint::TJointDesc::pBody1 = pBody1;
					IJoint::TJointDesc::pBody2 = pBody2;
					IPulleyJoint::TPulleyJointDesc::groundAnchor1 = groundAnchor1;
					IPulleyJoint::TPulleyJointDesc::groundAnchor2 = groundAnchor2;
					pBody1->GetLocalPoint(anchor1, localAnchor1);
					pBody2->GetLocalPoint(anchor2, localAnchor2);
					length1 = sqrtf(anchor1.distanceSqr(groundAnchor1));
					length2 = sqrtf(anchor2.distanceSqr(groundAnchor2));
					IPulleyJoint::TPulleyJointDesc::ratio = ratio;
					float C = length1 + ratio * length2;
					maxLength1 = C - ratio * MIN_LENGTH;
					maxLength2 = (C - MIN_LENGTH) / ratio;
				}


				/// The first ground anchor in world coordinates. This point never moves.
				Point2D_Int groundAnchor1;

				/// The second ground anchor in world coordinates. This point never moves.
				Point2D_Int groundAnchor2;

				/// The local anchor point relative to body1's origin.
				Point2D_Int localAnchor1;

				/// The local anchor point relative to body2's origin.
				Point2D_Int localAnchor2;

				/// The a reference length for the segment attached to body1.
				float length1;

				/// The maximum length of the segment attached to body1.
				float maxLength1;

				/// The a reference length for the segment attached to body2.
				float length2;

				/// The maximum length of the segment attached to body2.
				float maxLength2;

				/// The pulley ratio, used to simulate a block-and-tackle.
				float ratio;
			};
			#pragma pack(pop)

			/// Returns the first ground anchor.
			/// @param[out] pos the first ground anchor in px.
			virtual HRESULT CALLBACK GetGroundAnchor1(OUT Point2D_Int &pos) const = 0;

			/// Returns the second ground anchor.
			/// @param[out] pos the second ground anchor in px.
			virtual HRESULT CALLBACK GetGroundAnchor2(OUT Point2D_Int &pos) const = 0;

			/// Returns the current length of the segment attached to body1.
			/// @param[out] length the current length in px.
			virtual HRESULT CALLBACK GetLength1(OUT float &length) const = 0;

			/// Returns the current length of the segment attached to body2.
			/// @param[out] length the current length in px.
			virtual HRESULT CALLBACK GetLength2(OUT float &length) const = 0;

			/// Returns the pulley ratio.
			/// @param[out] ratio the pulley ratio.
			virtual HRESULT CALLBACK GetRatio(OUT float &ratio) const = 0;
		};

		//-----------------------------------------------------------------------
		//IGearJoint//
		//-----------------------------------------------------------------------

		// {B16EB899-674E-4b81-9F64-E08E452BCC29}
		static const GUID IID_IGearJoint = 
		{ 0xb16eb899, 0x674e, 0x4b81, { 0x9f, 0x64, 0xe0, 0x8e, 0x45, 0x2b, 0xcc, 0x29 } };

		/// A gear joint.
		/** Is used to connect two joints together. Either joint can be a revolute 
			or prismatic joint. You specify a gear ratio to bind the motions together:
			coordinate1 + ratio * coordinate2 = constant.
			The ratio can be negative or positive. If one joint is a revolute joint
			and the other joint is a prismatic joint, then the ratio will have units
			of length or units of 1/length.
		*/
		interface __declspec(uuid("{B16EB899-674E-4b81-9F64-E08E452BCC29}")) IGearJoint: public IJoint
		{
			/// Gear joint descriptor.
			/** This definition requires two existing revolute or prismatic joints
				(any combination will work). 
				The provided joints must attach a dynamic body to a static body.
			*/
			#pragma pack(push, 1)
			struct TGearJointDesc : public IJoint::TJointDesc
			{
				TGearJointDesc() : IJoint::TJointDesc()
				{
					type = e_gearJoint;
					ratio = 1.0f;
				}

				/// The first revolute/prismatic joint attached to the gear joint.
				/// @warning After creating joint reference will be incremented.
				IJoint *pJoint1;

				/// The second revolute/prismatic joint attached to the gear joint.
				/// @warning After creating joint reference will be incremented.
				IJoint *pJoint2;

				/// The gear ratio.
				/// @see IGearJoint.
				float ratio;
			};
			#pragma pack(pop)

			/// Returns the gear ratio.
			/// @param[out] ratio the gear ratio.
			virtual HRESULT CALLBACK GetRatio(OUT float &ratio) const = 0;

			/// Returns the first joint.
			/// @note Increments reference.
			/// @param[out] pJoint the first joint.
			virtual HRESULT CALLBACK GetJoint1(OUT IJoint *&pJoint) const = 0;

			/// Returns the second joint.
			/// @note Increments reference.
			/// @param[out] pJoint the second joint.
			virtual HRESULT CALLBACK GetJoint2(OUT IJoint *&pJoint) const = 0;
		};

		//-----------------------------------------------------------------------
		//IMouseJoint//
		//-----------------------------------------------------------------------

		// {F2FCFA15-8065-45f8-BF82-4A33A5B142D2}
		static const GUID IID_IMouseJoint = 
		{ 0xf2fcfa15, 0x8065, 0x45f8, { 0xbf, 0x82, 0x4a, 0x33, 0xa5, 0xb1, 0x42, 0xd2 } };

		/// A mouse joint.
		/** Used to make a point on a body track a specified world point.
			This a soft constraint with a maximum force.
			This allows the constraint to stretch and without applying huge forces.
		*/
		/// @warning Works only with collidable bodies.
		interface __declspec(uuid("{F2FCFA15-8065-45f8-BF82-4A33A5B142D2}")) IMouseJoint : public IJoint
		{
			/// Mouse joint descriptor.
			/** This requires a world target point, tuning parameters,
				and the time step.
			*/
			#pragma pack(push, 1)
			struct TMouseJointDesc : public IJoint::TJointDesc
			{
				TMouseJointDesc() : IJoint::TJointDesc()
				{
					maxForce = 0.0f;
					frequencyHz = 5.0f;
					dampingRatio = 0.7f;
					timeStep = 1.0f;
				}

				/// The initial world target point. This is assumed
				/// to coincide with the body anchor initially.
				Point2D_Int target;

				/// The maximum constraint force that can be exerted
				/// to move the candidate body. Usually you will express
				/// as some multiple of the weight (multiplier * mass * gravity).
				float maxForce;

				/// The response speed.
				float frequencyHz;

				/// The damping ratio. 0 = no damping, 1 = critical damping.
				float dampingRatio;

				/// The time step used in the simulation.
				float timeStep;
			};
			#pragma pack(pop)

			virtual HRESULT CALLBACK SetTarget(const Point2D_Int &target) = 0;
			virtual HRESULT CALLBACK GetTarget(OUT Point2D_Int &target) const = 0;
		};

		//-----------------------------------------------------------------------
		//IContactListener//
		//-----------------------------------------------------------------------

		// {44D65CAC-36B7-4435-B9AC-446CB9FAFDE0}
		static const GUID IID_IContactListener = 
		{ 0x44d65cac, 0x36b7, 0x4435, { 0xb9, 0xac, 0x44, 0x6c, 0xb9, 0xfa, 0xfd, 0xe0 } };

		/// The contact listner.
		interface __declspec(uuid("{44D65CAC-36B7-4435-B9AC-446CB9FAFDE0}")) IContactListener : public IDPEObject
		{
			/// Contact point state.
			enum EContactState
			{
				e_contactAdded,					///< contact point has been added.
				e_contactPersisted,				///< contact point presists.
				e_contactRemoved,				///< contact point has been removed.
			};

			/// Contact point information during time step.
			#pragma pack(push, 1)
			struct TContactPoint
			{
				IObject		*pShape1;		///< the first shape. Do not release it.
				IObject		*pShape2;		///< the second shape. Do not release it.
				Point2D_Int		position;		///< position in world coordinates.
				Vector2D_Float	velocity;		///< velocity of point on body2 relative to point on body1 (pre-solver).
				Vector2D_Float	normal;			///< points from shape1 to shape2.
				EContactState	state;			///< contact state. @see IContactListner::EContactState.

				TContactPoint()
				{
					pShape1 = NULL;
					pShape2 = NULL;
				}
			};

			#pragma pack(pop)

			virtual HRESULT CALLBACK NextPointStep(OUT TContactPoint &point) = 0;
			virtual HRESULT CALLBACK PrevPointStep(OUT TContactPoint &point) = 0;
			virtual HRESULT CALLBACK GetCountStep(OUT int &count) = 0;	

			virtual HRESULT CALLBACK NextPoint(OUT TContactPoint &point) = 0;
			virtual HRESULT CALLBACK PrevPoint(OUT TContactPoint &point) = 0;
			virtual HRESULT CALLBACK GetCount(OUT int &count) = 0;	
		};

		//-----------------------------------------------------------------------
		//ICollisionWorld//
		//-----------------------------------------------------------------------

		/// Box2D world interface wrapper.
		/** Used to control Box2D world. Do not try to query this interface.*/
		interface ICollisionWorld
		{

			/// Creates Box2D world.
			/// @param leftTop a left top point of bounding box that completely encompasses all your shapes.
			/// @param rightBottom a left top point of bounding box that completely encompasses all your shapes.
			/// @param gravity the world gravity vector.
			/// @param doSleep improve performance by not simulating inactive bodies.
			/// @param amendment an amendment for scale factor. @see ICollisionWorld::GetBox2DScaleFactor().
			virtual HRESULT CALLBACK CreateBox2DWorld(const Point2D_Int &leftTop, const Point2D_Int &rightBottom, const Vector2D_Float &gravity, bool doSleep, float amendment = 2.0f) = 0;

			/// Returns contact listener.
			/// @see IContactListener.
			/// @param[out] pListener the contact listener.
			virtual HRESULT CALLBACK GetContactListener(OUT IContactListener *&pListener) const = 0;

			/// Changes global gravity vector.
			/// @param[in] gravity a new gravity vector.
			virtual HRESULT CALLBACK SetGravity(const Vector2D_Float &gravity) = 0;

			/// Query the world for all shapes that potentially overlap the
			/// provided AABB.
			/// @warning works only with collidable body, shapes.
			/// @param leftTop a left top point of bounding box query.
			/// @param rightBottom a left top point of bounding box query.
			/// @param[out] pIter an iterator of objects 2d that contain shapes.
			/// @param maxCount the max count of the shapes array.
			virtual HRESULT CALLBACK Query(const Point2D_Int &leftTop, const Point2D_Int &rightBottom, int maxCount, OUT IIterator *&pIter) = 0;

			/// Returns the scale factor.
			/**	Is used to convert from render units(px) to Box2D (meters) of MKS(meter-kg-second).
				Aplied only on dimentions, vectors and velocity. Others use MKS.
				By default scale factor is max(screen_width, screen_height) / ( 10.0f * amendment )
				and amendment is 2.0f.
			*/
			/// @param[out] factor the scale factor.
			virtual HRESULT CALLBACK GetBox2DScaleFactor(OUT float &factor) = 0;

			/// Creates collidable body 2D.
			/// @note To create body you should create Box2D world first.
			/// @warning ICollidableBosy2D is no a part of Manager2D attach system.
			/// @param name the name of body.
			/// @param pBodyDesc the descriptor to create body. Not null.
			/// @param[out] pBody the created body.
			virtual HRESULT CALLBACK CreateCollidableBody(const char *name, IBody::TBodyDesc *pBodyDesc, OUT IBody *&pBody) = 0;

			/// Returns collidable body by name.
			/// @note If there is no body with the name returns E_FAIL and NULL.
			/// @param name the name of body to get.
			/// @param[out] pBody the collidable body.
			virtual HRESULT CALLBACK GetCollidableBody(const char *name, OUT IBody *&pBody) = 0;

			/// Creates distance joint.
			/// @see IDistanceJoint.
			/// @param name the name of joint.
			/// @param[in] pJointDesc the descriptor to create joint. Not null.
			/// @param[out] pJoint the created joint.
			virtual HRESULT CALLBACK CreateDistanceJoint(const char *name, IN IDistanceJoint::TDistanceJointDesc *pJointDesc, OUT IDistanceJoint *&pJoint) = 0;

			/// Creates revolute joint.
			/// @see IRevoluteJoint.
			/// @param name the name of joint.
			/// @param[in] pJointDesc the descriptor to create joint. Not null.
			/// @param[out] pJoint the created joint.
			virtual HRESULT CALLBACK CreateRevoluteJoint(const char *name, IN IRevoluteJoint::TRevoluteJointDesc *pJointDesc, OUT IRevoluteJoint *&pJoint) = 0;

			/// Creates prismatic joint.
			/// @see IPrismaticJoint.
			/// @param name the name of joint.
			/// @param[in] pJointDesc the descriptor to create joint. Not null.
			/// @param[out] pJoint the created joint.
			virtual HRESULT CALLBACK CreatePrismaticJoint(const char *name, IN IPrismaticJoint::TPrismaticJointDesc *pJointDesc, OUT IPrismaticJoint *&pJoint) = 0;

			/// Creates pulley joint.
			/// @param name the name of joint.
			/// @see IPulleyJoint.
			/// @param[in] pJointDesc the descriptor to create joint. Not null.
			/// @param[out] pJoint the created joint.
			virtual HRESULT CALLBACK CreatePulleyJoint(const char *name, IN IPulleyJoint::TPulleyJointDesc *pJointDesc, OUT IPulleyJoint *&pJoint) = 0;

			/// Creates gear joint.
			/// @note First and second joints can be of revolute/prismatic types.
			/// @see IGearJoint.
			/// @param name the name of joint.
			/// @param[in] pJointDesc the descriptor to create joint. Not null.
			/// @param[out] pJoint the created joint.
			virtual HRESULT CALLBACK CreateGearJoint(const char *name, IN IGearJoint::TGearJointDesc *pJointDesc, OUT IGearJoint *&pJoint) = 0;

			/// Creates mouse joint.
			/// @note Doesn't store joint in 2d manager container.
			/// @see IMouseJoint.
			/// @param name the name of joint.
			/// @param[in] pJointDesc the descriptor to create joint. Not null.
			/// @param[out] pJoint the created joint.
			virtual HRESULT CALLBACK CreateMouseJoint(const char *name, IMouseJoint::TMouseJointDesc *pJointDesc, OUT IMouseJoint *&pJoint) = 0;

			/// Returns joint by name.
			/// @note If there is no joint with the name returns E_FAIL and NULL.
			/// @param name the name of joint to get.
			/// @param[out] pJoint the joint.
			virtual HRESULT CALLBACK GetJoint(const char *name, OUT IJoint *&pJoint) = 0;

			/// Box2D debug draw flags.
			enum EDebugDraw
			{
				e_shapeBit				= 0x0001, ///< draw shapes
				e_jointBit				= 0x0002, ///< draw joint connections
				e_coreShapeBit			= 0x0004, ///< draw core (TOI) shapes
				e_aabbBit				= 0x0008, ///< draw axis aligned bounding boxes
				e_obbBit				= 0x0010, ///< draw oriented bounding boxes
				e_pairBit				= 0x0020, ///< draw broad-phase pairs
				e_centerOfMassBit		= 0x0040, ///< draw center of mass frame
			};

			/// Set Box2D debug draw flags.
			/// @note By default is set: shapes, joint connections, center of mass frame
			/// @param flags the new flags.
			/// @see EDebugDraw.
			virtual HRESULT CALLBACK SetDebugDrawFlags(int flags) = 0;
			
			/// Get the drawing flags.
			/// @param[out] flags the flags.
			/// @see EDebugDraw.
			virtual HRESULT CALLBACK GetDebugDrawFlags(OUT int &flags) = 0;

			/// Add flags.
			/// @param flags the flgs to append.
			/// @see EDebugDraw.
			virtual HRESULT CALLBACK AppendDebugDrawFlags(int flags) = 0;

			/// Clear all debug draw flags.
			virtual HRESULT CALLBACK ClearDebugDrawFlags() = 0;
		};
	}

}

#include "UNDEF_FOR_BORLANDC"

#endif