/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		11.11.2008
LastModified:	...

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
! warning camera 2d SetScale doesn't work.
-----------------------------------------------------------------------*/
#ifndef _2D_INTERFACES_H_INCLUDED_
#define _2D_INTERFACES_H_INCLUDED_

#include "IDPEObject.h"
#include "2DCollision.h"

namespace DPE_Scenes
{
	namespace _2D
	{

		/// DPE events type enumeration.
		/** Used to subscribe on events in GUI and ISpriteEx. */
		enum DPE_EVENT_TYPE
		{
			ET_ON_MOUSE_MOVE = 0,	///< activate on mouse move. Used in ISpriteEx and GUI.
			ET_ON_MOUSE_DOWN,		///< activate on mouse down. Used in ISpriteEx and GUI.
			ET_ON_MOUSE_UP,			///< activate on mouse up. Used in ISpriteEx and GUI.
			ET_ON_MOUSE_PRESS,		///< activate on mouse press. Used in ISpriteEx and GUI.
			ET_ON_MOUSE_ENTER,		///< activate on mouse enter in object area. Used in ISpriteEx and GUI.
			ET_ON_MOUSE_LEAVE,		///< activate on mouse leave object area. Used in ISpriteEx and GUI.
			ET_ON_CLICK,			///< activate on mouse click. Used in ISpriteEx and GUI.
			ET_ON_DOUBLE_CLICK,		///< activate on mouse double click. Used in ISpriteEx and GUI.
			ET_ON_MOUSE_WHEEL,		///< activate on mouse wheel. Used in ISpriteEx and GUI.

			ET_ON_KEY_DOWN,			///< activate on key down. Used in ISpriteEx and GUI.
			ET_ON_KEY_UP,			///< activate on key up. Used in ISpriteEx and GUI.
			ET_ON_KEY_PRESS,		///< activate on key press. Used in ISpriteEx and GUI.

			ET_ON_DRAG,				///< activate on object drag. Used in ISpriteEx and GUI.
			ET_ON_RESIZE,			///< activate on resize. Used in GUI.
			ET_ON_FOCUS,			///< activate on focus. Used in GUI.
			ET_ON_FOCUS_LOST,		///< activate on focus lost. Used in GUI.
			ET_ON_DESTROY,			///< activate on destroy. Used in GUI.
			ET_ON_CLOSE				///< activate on close. Used in IDPEGUIWindow.
		};

		// Forward declarations
		interface IObjectBase;

		//-----------------------------------------------------------------------
		//IIterator//
		//-----------------------------------------------------------------------
		// {6B663BA4-428E-410c-B733-032FC0087115}
		static const GUID IID_IIterator = 
		{ 0x6b663ba4, 0x428e, 0x410c, { 0xb7, 0x33, 0x3, 0x2f, 0xc0, 0x8, 0x71, 0x15 } };

		/// An iterator for a list of IObjects.
		/** Used for over a list of IObjects. On create the index is -1.
			So to start pass list call next. See below.	
		*/
		/**	@code
			IObject2DIterator *pIter = NULL;
			if ( SUCCEEDED(g_pMan2D->Query(left_top, right_bottom, 10, pIter)) && pIter != NULL )
			{
				IObject *pObject = NULL;
				while ( SUCCEEDED(pIter->Next()) && SUCCEEDED(pIter->Get(pObject)) && pObject != NULL )
				{
					// do smth with pObject
					...
					SAFE_RELEASE(pObject);
				}
			}
			SAFE_RELEASE(pIter);
			@endcode
		*/
		interface __declspec(uuid("{6B663BA4-428E-410c-B733-032FC0087115}")) IIterator: public IDPEObject
		{
			/// Moves the iterator to next object
			/// If iterator is out of range returns S_FALSE
			virtual HRESULT CALLBACK Next() = 0;

			/// Moves iterator to previous object
			/// If iterator is out of range returns S_FALSE
			virtual HRESULT CALLBACK Prev() = 0;

			/// Gets IObjects count
			/// @param count the number of objects
			virtual HRESULT CALLBACK Count(OUT unsigned int &count) = 0;

			/// Gets IObject of iterator
			/// @note Increments reference.
			/// @param[out] pObject the interface of object
			/// @warning If iterator is out of range returns S_FALSE and pObject == NULL;
			virtual HRESULT CALLBACK Get(OUT IObjectBase *&pObject) = 0;
		};

		//-----------------------------------------------------------------------
		//IObjectBase//
		//-----------------------------------------------------------------------
		// {14DFAEB5-A8ED-45d0-AEDC-BC8C68154271}
		static const GUID IID_IObjectBase = 
		{ 0x14dfaeb5, 0xa8ed, 0x45d0, { 0xae, 0xdc, 0xbc, 0x8c, 0x68, 0x15, 0x42, 0x71 } };

		/// A base interface of 2d object.
		/** Includes such properties as visible status, position, attach system and z order.
		*/
		interface __declspec(uuid("{14DFAEB5-A8ED-45d0-AEDC-BC8C68154271}")) IObjectBase: public IDPEObject
		{
			/// Setups object visibility.
			/// @param visible the visible flag.
			virtual HRESULT CALLBACK SetVisible(bool visible) = 0;

			/// Obtain object visiblity.
			/// @param[out] the object visibility.
			virtual HRESULT CALLBACK GetVisible(OUT bool &visible) = 0;

			/// Setups object local position.
			/// @remark Setups relative position. For instance, if object is attached
			/// to another then method sets local position relative to parent.
			/// @param x the x local coordinate.
			/// @param y the y local coordinate.
			virtual HRESULT CALLBACK SetPosition(int x, int y) = 0;

			/// Setups object local position.
			/// @param[in] pos the local position.
			virtual HRESULT CALLBACK SetPosition(const Point2D_Int &pos) = 0;

			/// Returns global position.
			/// @param[out] pos the world position.
			virtual HRESULT CALLBACK GetPosition(OUT Point2D_Int &pos) = 0;

			/// Returns local position.
			/// @note If object is attached to another local position is the global one.
			/// @param pos the local position.
			virtual HRESULT CALLBACK GetOffset(OUT Point2D_Int &pos) = 0;

			/// Attach an object.
			/// @remark An object may be one inherited from IObject.
			/// By default object is attached to Manager2D.
			/// @note Increments an object reference.
			/// @warning If object isn't attached to some one it doesn't updating and
			/// drawing by Manager2D. Therefor, you should manually control it.
			/// The recommended way is to use attaching system.
			/// @see ISprite.
			/// @see ISpriteEx.
			/// @see IEmitter2D.
			/// @see IObject::Detach()
			/// @param pObject the childe object.
			virtual HRESULT CALLBACK Attach(IN IObjectBase *pObject) = 0;

			/// Detach an object.
			/// @note If an object is not attached returns E_FAIL.
			/// Decrements child reference.
			/// @see IObject::Attach().
			/// @param pObject a object to detach.
			virtual HRESULT CALLBACK Detach(IObjectBase *pObject) = 0;

			/// Detach all children.
			/// @note Decrements child reference.
			/// @see IObject::Attach().
			virtual HRESULT CALLBACK DetachAll( void ) = 0;

			/// Detach this object from parent.
			/// @see IObject::Attach().
			virtual HRESULT CALLBACK DetachFromParent( void ) = 0;

			/// Setups parent.
			/// @warning do not use this method.
			/// It is for internal purposes.
			virtual HRESULT CALLBACK SetParent(IN IObjectBase *pObject) = 0;

			/// Returns parent.
			/// @note Increments reference.
			/// @param[out] pObject the parent object.
			virtual HRESULT CALLBACK GetParent(OUT IObjectBase *&pObject) = 0;

			/// Is the object is a child of this.
			virtual HRESULT CALLBACK IsChild(const IObjectBase *pObject, OUT bool &child) = 0;

			/// Returns children count.
			/// @param[out] count the children numbres.
			virtual HRESULT CALLBACK GetChildrenCount(OUT unsigned int &count) = 0;

			/// Returns an iterator to list of children.
			/// @note Iterator increments children references.
			/// @see IObject2DIterator.
			/// @param[out] pIterator the iterator to list.
			virtual HRESULT CALLBACK GetChildren(OUT IIterator *&pIterator) = 0;

			/// Setups z order of object in render stack.
			/// In the range from 0 (the top) to count - 1 (the bottom).
			/// @note It works in range of children attached to the object.
			/// First parent then children from count - 1 to 0.
			/// To reverse order use IObject::ReverseOrder().
			/// @see IObject::ReverseOrder().
			/// @param idx the index of z order.
			virtual HRESULT CALLBACK SetZOrder(unsigned int idx) = 0;

			/// Returns z order of object in render stack.
			virtual HRESULT CALLBACK GetZOrder(OUT unsigned int &idx) = 0;

			/// Reverse z order.
			/// @see IObject::SetZOrder().
			virtual HRESULT CALLBACK ReverseOrder() = 0;

			/// Returns if z order is reversed.
			/// @see IObject::ReverseOrder().
			/// @param[out] reversed the flag of if z order reversed.
			virtual HRESULT CALLBACK GetReverseOrder(OUT bool &reversed) = 0;

			/// Manual draw of object.
			/// @note If you do not want use cliping region set it to NULL.
			/// @warning Use this if you manually control object.
			/// Otherwise object will be rendered twice.
			/// @see IObject::Attach().
			/// @param clipRect the pointer of cliping region.
			virtual HRESULT CALLBACK Draw(Rect2D_Int *clipRect = NULL) = 0;

			/// Manual update.
			/// @note By default use speed = 1.0f.
			/// That corresponds to one call in 33 milliseconds.
			/// @warning Use this if you manually control object.
			/// Otherwise object will be updated twice.
			/// @param speed the speed update.
			virtual HRESULT CALLBACK Update(float speed) = 0;
			
			/// Update transformation of object and attached children.
			/// @warning Do not use this. It is for internal purposes.
			virtual HRESULT CALLBACK UpdateTransformation( void ) = 0;
		};


		//-----------------------------------------------------------------------
		//IObject//
		//-----------------------------------------------------------------------
		// {2761779A-3B96-4d3b-A8DA-5FA8731457C3}
		static const GUID IID_IObject = 
		{ 0x2761779a, 0x3b96, 0x4d3b, { 0xa8, 0xda, 0x5f, 0xa8, 0x73, 0x14, 0x57, 0xc3 } };

		/// A main interface of 2d object.
		/** Used for empty (invisible) transforms and connections. Also is a shape for ICollidableBody2D.
			If empty object is used as a shape of collidable object then rotations isn't accepted.
			To destroy object just call free: from 2d manager, collidable body (if presist),
			parent. To completely destroy release all references in your program.
		*/
		/// @see IObject2DBase.
		/// @see ICollisionShape2D.
		interface __declspec(uuid("{2761779A-3B96-4d3b-A8DA-5FA8731457C3}")) IObject : 
			public IObjectBase, public ICollisionShape, public IClone, public IFree
		{
			virtual HRESULT CALLBACK SetCameraOffsetScale(const Vector2D_Float &offset) = 0;
			virtual HRESULT CALLBACK GetCameraOffsetScale(OUT Vector2D_Float &offset) const = 0;
		};

		//-----------------------------------------------------------------------
		//ISprite//
		//-----------------------------------------------------------------------
		// {739B418B-4EF9-4a9d-AAB9-73EB41EDF4AB}
		static const GUID IID_ISprite = 
		{ 0x739b418b, 0x4ef9, 0x4a9d, { 0xaa, 0xb9, 0x73, 0xeb, 0x41, 0xed, 0xf4, 0xab } };

		/// Base sprite with animation.
		/**	To use sprite with animation setup texture with frames.
			@see ISprite::SetTexture().
			Frames numeration goes from left to right, from top to bottom order.
		*/
		interface __declspec(uuid("{739B418B-4EF9-4a9d-AAB9-73EB41EDF4AB}")) ISprite: public IObject
		{
			/// Setups texture
			/// @remark To use sprite as animated setup texture with frames.
			/// @param name the texture resource name.
			/// @param countX the frames count by horizontal. Default is 1.
			/// @param countY the frames count by vertical. Default is 1.
			virtual HRESULT CALLBACK SetTexture(const char *name, unsigned int countX = 1, unsigned int countY = 1) = 0;

			/// Returns texture used in sprite if there is such
			virtual HRESULT CALLBACK GetTexture(OUT ITexture *&texture) const = 0;

			/// Is animation going on?
			/// @param[out] bAnim flag og animation.
			virtual HRESULT CALLBACK IsAnimation(OUT bool &bAnim) const = 0;

			/// Indicate sprite to loop animation.
			/// @param looped the looped animation flag.
			virtual HRESULT CALLBACK Loop(bool looped) = 0;

			/// Setups the sprite animation speed.
			/// @note Default speed is 1.0f.
			/// @param speed a new animation speed.
			virtual HRESULT CALLBACK SetSpeed(float speed) = 0;

			/// Setups frames count.
			virtual HRESULT CALLBACK SetFramesCount(unsigned int countX = 1, unsigned int countY = 1) = 0;

			/// Returns frames count.
			virtual HRESULT CALLBACK GetFramesCount(OUT unsigned int &countX, OUT unsigned int &countY) = 0;
			
			/// Setups frames limits.
			/// @param first the lower limit.
			/// @param last the upper limit.
			virtual HRESULT CALLBACK SetFrames(int first = 0, int last = -1) = 0;

			/// Setups current frame.
			/// @param idx the index of selected frame.
			virtual HRESULT CALLBACK SetCurrentFrame(int idx) = 0;

			/// Returns current frame.
			/// @param[out] idx the index of current frame.
			virtual HRESULT CALLBACK GetCurrentFrame(OUT int &idx) const = 0;

			/// Setups texture coordinates and polygon verteces.
			virtual HRESULT CALLBACK SetVerteces(size_t vcount, const Vertex2D_RGBA vertices[]) = 0;

			/// Unset texture coordinates and polygon verteces.
			virtual HRESULT CALLBACK UnsetVerteces() = 0;

			/// Setups verteces colors.
			/// @note Common alpha is average of 5 verteces.
			/// @param[in] color a new colors of verteces.
			virtual HRESULT CALLBACK SetColor(const ColorRGBA (&color)[4]) = 0;

			/// Setups alpha.
			/// @note Alpha will be common for all verteces.
			/// @param alpha a new alpha.
			virtual HRESULT CALLBACK SetAlpha(unsigned char alpha) = 0;

			/// Returns alpha.
			/// @param[out] alpha the sprite alpha.
			virtual HRESULT CALLBACK GetAlpha(OUT unsigned char &alpha) const = 0;

			/// Setups blend mode.
			/// @remark If param is false - blend one. Otherwise - one minus src alpha.
			/// By default is false.
			/// @param add_back the blend mode. 
			virtual HRESULT CALLBACK BlendMode(bool add_back) = 0;
			
			/// Setups a new sprite dimention.
			/// @param width a new sprite width.
			virtual HRESULT CALLBACK SetDimension(int width, int height) = 0;

			/// Returns the sprite dimention.
			/// @param dimension the sprite dimention.
			virtual HRESULT CALLBACK GetDimension(OUT Dimension2D_UInt &dimension) const = 0;
			
			/// Setups rotation in radians.
			/// @param angle the new world rotation angle of the sprite in radians.
			/// @param center the point in local system.
			virtual HRESULT CALLBACK SetAngle(float angle, const Point2D_Int &center) = 0;

			/// Setups angle of rotation.
			/// @see SetAngle.
			virtual HRESULT CALLBACK SetAngle(float angle) = 0;

			/// Returns rotation in radians.
			/// @param angle the angle of the sprite in radians.
			virtual HRESULT CALLBACK GetAngle(OUT float &angle) const = 0;

			/// Returns rotation center in relative coordiantes.
			virtual HRESULT CALLBACK GetRotCenter(OUT Point2D_Int &center) const = 0;
				
			/// Is sprite is fliped by x.
			virtual HRESULT CALLBACK FlipX() = 0;

			virtual HRESULT CALLBACK IsFlipX(OUT bool &flip) const = 0;

			/// Flip sprite by y.
			virtual HRESULT CALLBACK FlipY() = 0;

			/// Is sprite is fliped by y.
			virtual HRESULT CALLBACK IsFlipY(OUT bool &flip) const = 0;
		};

		//-----------------------------------------------------------------------
		//ISpriteEx//
		//-----------------------------------------------------------------------

		// {01A2BA1E-C4C0-44ed-BC14-F56D52AE86E4}
		static const GUID IID_ISpriteEx = 
		{ 0x1a2ba1e, 0xc4c0, 0x44ed, { 0xbc, 0x14, 0xf5, 0x6d, 0x52, 0xae, 0x86, 0xe4 } };

		/// Sprite with mouse and keyboard callbacks.
		/** Events do not work if the sprite is a part of collidable body.
			@see ICollidableBody2D.
			@see IMouseJoint.
		*/
		interface __declspec(uuid("{01A2BA1E-C4C0-44ed-BC14-F56D52AE86E4}")) ISpriteEx: public ISprite
		{
			/// Indicate to move sprite by mouse only by rect.
			/// @param dragged the flag of sprite drag posibility.
			virtual HRESULT CALLBACK SetDragged(bool dragged) = 0;

			/// Appends an event subscriber.
			/// @see DPE_EVENT_TYPE.
			/// @param type the type of event to subscribe.
			/// @param subscriber a new subscriber.
			virtual HRESULT CALLBACK AddListerner(DPE_EVENT_TYPE type, void (*subscriber)(IEvent *)) = 0;

			/// Deelte the event subscriber.
			/// @see DPE_EVENT_TYPE.
			/// @param type the type of event to subscribe.
			/// @param subscriber the subscriber to delete.
			virtual HRESULT CALLBACK RemoveListerner(DPE_EVENT_TYPE type, void (*subscriber)(IEvent *)) = 0;	
		};

		//-----------------------------------------------------------------------
		//IEmitter2D//
		//-----------------------------------------------------------------------

		// {9739417B-E9E7-43d0-8D28-AFDFC2DB753E}
		static const GUID IID_IEmitter2D = 
		{ 0x9739417b, 0xe9e7, 0x43d0, { 0x8d, 0x28, 0xaf, 0xdf, 0xc2, 0xdb, 0x75, 0x3e } };

		/// 2D particle emitter.
		/** If emitter is a shape of collidable object then rotations isn't accepted.
			It will work as in the editor.
		*/
		interface __declspec(uuid("{9739417B-E9E7-43d0-8D28-AFDFC2DB753E}")) IEmitter2D: public IObject
		{
			virtual HRESULT CALLBACK SetAngle(float angle) = 0;
			virtual HRESULT CALLBACK GetAngle(OUT float &angle) const = 0;

			/// Change emitter sprites scale.
			/// @param fScalar a new scale.
			virtual HRESULT CALLBACK SetScale(float fScalar) = 0;

			/// Returns emitter scale.
			/// @param[out] fScalar the current scale.
			virtual HRESULT CALLBACK GetScale(OUT float &fScalar) const = 0;

			/// Setups an emitter alpha.
			/// @param alpha a new alpha.
			virtual HRESULT CALLBACK SetAlpha(unsigned char alpha) = 0;

			/// Returns the emitter alpha.
			/// @param[out] alpha the emitter alpha.
			virtual HRESULT CALLBACK GetAlpha(OUT unsigned char &alpha) = 0;

			/// Returns active status.
			/// @param[out] active the status.
			virtual HRESULT CALLBACK IsActive(OUT bool &active) = 0;

			/// Reset emitter animation.
			virtual HRESULT CALLBACK Reset(void) = 0;

			/// Set emitter animation speed.
			/// @note Default speed is 1.0f.
			/// @param fSpeed the new speed.
			virtual HRESULT CALLBACK SetSpeed(float fSpeed) = 0;
		};


		//-----------------------------------------------------------------------
		//ICamera2D//
		//-----------------------------------------------------------------------
		// {7CE77613-1B44-4b41-A2E2-E61132AEEAF8}
		static const GUID IID_ICamera2D = 
		{ 0x7ce77613, 0x1b44, 0x4b41, { 0xa2, 0xe2, 0xe6, 0x11, 0x32, 0xae, 0xea, 0xf8 } };

		/// 2D camera.
		/** Used to work with view area. The world camera's origin is the center of screen.
			It doesn't affect on objects positions. Also it works with 2d primitives. @see IManager2D.
			There is only one copy of the camera instance - singleton.
		*/
		interface __declspec(uuid("{7CE77613-1B44-4b41-A2E2-E61132AEEAF8}")) ICamera2D: public IDPEObject
		{
			/// Setups camera activity.
			/// @note By default camera works.
			/// @param enable the flag of enable/disable camera activity.
			virtual HRESULT CALLBACK SetActive(bool enable) = 0;

			/// Setups a new camera position.
			/// @param[in] pos a new camera position.
			virtual HRESULT CALLBACK SetPosition(const Point2D_Int &pos) = 0;

			/// Returns the camera position. Camera center is in the center of the screen.
			/// @param[out] pos the camera position.
			virtual HRESULT CALLBACK GetPosition(OUT Point2D_Int &pos) const = 0;

			/// Returns the camera offset from (0,0).
			/// @warning If camera is disabled returns (0,0).
			/// @param[out] pos the camera offset.
			virtual HRESULT CALLBACK GetOffset(OUT Point2D_Int &pos) const = 0;
			
			/// Setups the camera zoom.
			/// @param fScale a new camera scale.
			//virtual HRESULT CALLBACK SetScale(float fScale) = 0;

			/// Returns the camera zoom.
			/// @param fScale the camera scale.
			//virtual HRESULT CALLBACK GetScale(OUT float &fScale) const = 0;

			/// Moves the camera to point.
			/// @param pos the camera target in world coordinates.
			virtual HRESULT CALLBACK MoveTo(const Point2D_Int &pos) = 0;

			/// Moves to object's origin.
			/// @note Use the method to attach the camera to 2d object.
			/// If target changes position the camera follows.
			/// To detach the camera use NULL pointer as param.
			/// @param[in] pObject a new camera target.
			virtual HRESULT CALLBACK MoveToObject(IN IObject *pObject) = 0;

			/// Setups the speed of camera movement.
			/// @param fSpeed a new camera speed.
			virtual HRESULT CALLBACK SetSpeed(float fSpeed) = 0;

			/// Returns the camera speed.
			/// @param[out] fSpeed the camera speed.
			virtual HRESULT CALLBACK GetSpeed(OUT float &fSpeed) const = 0;

			/// Is camera moving.
			/// @param bMoving the flag.
			virtual HRESULT CALLBACK IsMoving(OUT bool &bMoving) const = 0;

			/// Returns screen resolution.
			/// @param[out] width the screen width.
			/// @param[out] height the screen height.
			virtual HRESULT CALLBACK GetScreenResolution(OUT int &width, OUT int &height) = 0;
		};

		//-----------------------------------------------------------------------
		//IManager2D//
		//-----------------------------------------------------------------------

		// {12A260DA-9F46-4cea-83FC-CFCEBB0C58D4}
		static const GUID IID_IManager2D = 
		{ 0x12a260da, 0x9f46, 0x4cea, { 0x83, 0xfc, 0xcf, 0xce, 0xbb, 0xc, 0x58, 0xd4 } };

		/// The 2d manager.
		/** There is one instance of object (singleton).
			Creates and contains 2d objects. Used for drawing 2d primitives.
			Containers of objects, sprites, sprites ex, emitters, collidable objects are differrent.
			So you can use the same names, for instance: for an object and a sprite.
			Joints are kept in one containers. So use differrent names to different types of joints.
			Mouse joint (@see IMouseJoint2D) is not stored in joint container.
		*/
		interface __declspec(uuid("{12A260DA-9F46-4cea-83FC-CFCEBB0C58D4}")) IManager2D: public IObjectBase, public ICollisionWorld
		{
			/// Draw a point.
			/// @param[in] point the point vertex.
			/// @param size point size. Default 1.0f.
			virtual HRESULT CALLBACK DrawPoint(const Point2D_RGBA &point, float size = 1.0f, bool bUseCamera = false) = 0;

			/// Draw a line.
			/// @param[in] p1 the first line vertex.
			/// @param[in] p2 the second line vertex.
			virtual HRESULT CALLBACK DrawLine(const Point2D_RGBA &p1, const Point2D_RGBA &p2, bool bUseCamera = false) = 0;

			/// Draw a rectangle.
			/// @param[in] rect the rectangle verteces.
			/// @param fill indicate rectangle fill.
			virtual HRESULT CALLBACK DrawRectangle(const Point2D_RGBA (&rect)[4], bool fill = false, bool bUseCamera = false) = 0;

			/// Draw a circle.
			/// @param[in] center the center point of circle. Contains circle color.
			/// @param radius the circle radius.
			/// @param vcount the circle accuracy.
			/// @param fill indicate circle fill.
			virtual HRESULT CALLBACK DrawCircle(const Point2D_RGBA &center, float radius, unsigned int vcount, bool fill = false, bool bUseCamera = false) = 0;

			/// Draw a ellipse.
			/// @param[in] center the center point of ellipse. Contains ellipse color.
			/// @param rx the ellipse radius on x.
			/// @param ry the circle radius on y.
			/// @param vcount the circle accuracy.
			/// @param fill indicate circle fill.
			virtual HRESULT CALLBACK DrawEllipse(const Point2D_RGBA &center, float rx, float ry, unsigned int vcount, bool fill = false, bool bUseCamera = false) = 0;

			/// Doesn't work.
			virtual HRESULT CALLBACK DrawCircleArc(const Point2D_RGBA &center, float radius, float angle1, float angle2, bool bUseCamera = false) = 0;

			/// Draw a polygon.
			/// @param vcount polygon verteces count.
			/// @param[in] vertices polygon verteces.
			/// @param fill indicate circle fill.
			virtual HRESULT CALLBACK DrawPolygon(unsigned int vcount, const Point2D_RGBA vertices[], bool fill = false, bool bUseCamera = false) = 0;

			/// Returns the instance of camera.
			/// @see ICamera2D.
			/// @param[out] pCamera2D the camera instance.
			virtual HRESULT CALLBACK GetCamera(OUT ICamera2D *&pCamera2D) = 0;

			/// Indicate to attache created objects to 2d manager.
			/// @remark By default enabled.
			/// @param enable the flag.
			virtual HRESULT CALLBACK AutoAttach(bool enable) = 0;

			/// Creates empty 2d object.
			/// @see IObject.
			/// @param name a name of new object.
			/// @param[out] pObject the object instance.
			virtual HRESULT CALLBACK CreateEmptyObject(const char *name, OUT IObject *&pObject) = 0;

			/// Returns empty 2d object.
			/// @note Increment object reference.
			/// If object doesn't exist then returns E_FAIL and param is NULL.
			/// @see IObject.
			/// @param name the name of object.
			/// @param[out] pSprite the object instance.
			virtual HRESULT CALLBACK GetEmptyObject(const char *name, OUT IObject *&pObject) = 0;

			/// Creates 2d sprite.
			/// @see ISprite.
			/// @param name a name of new object.
			/// @param[out] pSprite the object instance.
			virtual HRESULT CALLBACK CreateSprite(const char *name, OUT ISprite *&pSprite) = 0;

			/// Returns 2d sprite.
			/// @note Increment object reference.
			/// If sprite doesn't exist then returns E_FAIL and param is NULL.
			/// @see ISprite.
			/// @param name the name of object.
			/// @param[out] pSprite the object instance.
			virtual HRESULT CALLBACK GetSprite(const char *name, OUT ISprite *&pSprite) = 0;

			/// Creates 2d sprite ex.
			/// @see ISpriteEx.
			/// @param name a name of new object.
			/// @param[out] pSprite the object instance.
			virtual HRESULT CALLBACK CreateSpriteEx(const char *name, OUT ISpriteEx *&pSprite) = 0;

			/// Returns 2d sprite ex.
			/// @note Increment object reference.
			/// If sprite doesn't exist then returns E_FAIL and param is NULL.
			/// @see ISpriteEx.
			/// @param name the name of object.
			/// @param[out] pEmitter2D the object instance.
			virtual HRESULT CALLBACK GetSpriteEx(const char *name, OUT ISpriteEx *&pEmitter2D) = 0;

			/// Creates 2d emitter.
			/// @note Be sure that emitter resource is not volume.
			/// @see IEmitter2D.
			/// @param name a name of new object.
			/// @param emitter the name of emitter resource.
			/// @param[out] pEmitter2D the object instance.
			virtual HRESULT CALLBACK CreateParticleEmitter(const char *name, const char* emitter, OUT IEmitter2D *&pEmitter2D) = 0;

			/// Returns 2d emitter.
			/// @note Increment object reference.
			/// If emitter doesn't exist then returns E_FAIL and param is NULL.
			/// @see IEmitter2D.
			/// @param name the name of object.
			/// @param[out] pEmitter2D the object instance.
			virtual HRESULT CALLBACK GetParticleEmitter(const char *name, OUT IEmitter2D *&pEmitter2D) = 0;

			//virtual HRESULT CALLBACK CreateSprite3D(const char *name, OUT IDPESprite3D *&pSprite) = 0;
			//virtual HRESULT CALLBACK GetSprite3D(const char *name, OUT IDPESprite3D *&pSprite) = 0;

			/*
				Manually remove emitters idle on update
			*/
			/// Removes inactive emitters.
			/// @remark Call this on update. Removes emitters from manager
			/// and parent if attached.
			/// @note If you have a reference in your program on emitter
			/// it will not destroied.
			virtual HRESULT CALLBACK RemoveInActiveEmitters() = 0;

			/// Indicate to remove inactive emitters.
			/// @remark The emitters that animation is not looped
			/// will be idle on update.
			/// To remove them and destroy set true.
			/// By default is true.
			/// @note If you have a reference in your program on emitter
			/// it will not destroied.
			/// @param remove the flag.
			virtual HRESULT CALLBACK RemoveIdleEmitters(bool remove) = 0;

			/// Indicate to change order of sprite ex on mouse select.
			/// @note The selected sprite ex will be moved to top.
			/// By defaul is false.
			/// @param change the flag.
			virtual HRESULT CALLBACK ChangeOrderOnSelect(bool change) = 0;

			virtual HRESULT CALLBACK SetMaxParticleEmitters(int count) = 0;
			virtual HRESULT CALLBACK SetMaxParticles(int count) = 0;
		};
	}

}
#endif