/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		20.03.2008
LastModified:	14.11.2008

Revision history:
	* Created by Mike
	* Revised by ...

Disctription:
-----------------------------------------------------------------------*/
#ifndef _IRESOURCE_MANAGER_H_INCLUDED_
#define _IRESOURCE_MANAGER_H_INCLUDED_

#include "IDPEObject.h"
#include "math\\aabb.h"
#include "DPETypes.h"
#include "IRenderer.h"
#include "enums.h"

using namespace RendererEnums;
using namespace RendererEnums::IStandardMaterial;
using namespace RendererEnums::IInstance;

namespace DPE_Scenes
{

#pragma pack(push, 1)
typedef struct
{
	DPE_OBJECT_TYPE	Type;
} TBaseResourceDesc;

typedef struct: public TBaseResourceDesc
{
	unsigned int	Width;
	unsigned int	Height;
	short			Levels;//! -1 all levels are used. 0 from the file 
	TEXTURE_FORMAT	Format;
	bool			Target;//! Is render target
} TTextureDesc;

typedef struct: public TBaseResourceDesc
{
	LIGHTING_MODEL	LightingModel;
	bool			AlphaTest;
	bool			Anisotropy;	
	bool			ShadowReceiver;
	const char*		TexturePath;
} TMaterialDesc;

typedef struct: public TBaseResourceDesc
{
	bool			ShadowCaster;
	TMaterialDesc	*Material;
} TMeshDesc;

typedef struct: public TBaseResourceDesc
{
} TFontDesc;

#pragma pack(pop)

//-----------------------------------------------------------------------
//IMesh//
//-----------------------------------------------------------------------
#define DEFAULT_MESH_CUBE "cube"
#define DEFAULT_MESH_PLANE "plane"
#define DEFAULT_MESH_SPHERE "sphere"

// {DB9C05D1-C829-4c17-A8DE-334A8D4B63F1}
static const GUID IID_IMesh = 
{ 0xdb9c05d1, 0xc829, 0x4c17, { 0xa8, 0xde, 0x33, 0x4a, 0x8d, 0x4b, 0x63, 0xf1 } };

interface __declspec(uuid("{DB9C05D1-C829-4c17-A8DE-334A8D4B63F1}")) IMesh: public IDPEObject
{
	virtual HRESULT CALLBACK GetAABB(OUT TAABB &aabb) = 0;
};
//-----------------------------------------------------------------------
//ITexture//
//-----------------------------------------------------------------------
#define DEFAULT_TEXTURE "default"

// {A48C05A9-0D1C-4ea7-8F12-7072EF765BDE}
static const GUID IID_ITexture = 
{ 0xa48c05a9, 0xd1c, 0x4ea7, { 0x8f, 0x12, 0x70, 0x72, 0xef, 0x76, 0x5b, 0xde } };

interface __declspec(uuid("{A48C05A9-0D1C-4ea7-8F12-7072EF765BDE}")) ITexture: public IDPEObject
{	
	virtual HRESULT CALLBACK DrawSimple(const Point2D_Int &pos, unsigned char alpha = 255) = 0;
	virtual HRESULT CALLBACK Draw(const Point2D_Int &pos, const Dimension2D_UInt &dim, unsigned char alpha = 255, bool flipX = false, bool flipY = false) = 0;
	virtual HRESULT CALLBACK DrawEx(const Point2D_Int &pos, const Dimension2D_UInt &dim, const ColorRGB &color, unsigned char alpha, float angle, const Point2D_Int &rotCenter, bool flipX = false, bool flipY = false) = 0;
	virtual HRESULT CALLBACK DrawExVertexColor(const Point2D_Int &pos, const Dimension2D_UInt &dim, const ColorRGBA (&color)[4], float angle, const Point2D_Int &rotCenter, bool flipX = false, bool flipY = false) = 0;

	virtual HRESULT CALLBACK GetWidth(OUT unsigned int &width) = 0;
	virtual HRESULT CALLBACK GetHeight(OUT unsigned int &height) = 0;
	
	virtual HRESULT CALLBACK GetRendererTextureInterface(OUT IUnknown *&pTexture) = 0;	
};
//-----------------------------------------------------------------------
//IMaterial//
//-----------------------------------------------------------------------
//! Default material name
#define DEFAULT_MATERIAL "default"
//! Number of layers that material can contain
#define MAX_TEXTURE_LAYERS 4

enum TEXTURE_LAYER
{
	TL_AMBIENT_DIFFUSE = 0,
	TL_SPECULAR,
	TL_NORMAL,
	TL_HEIGHT
};

// {10200473-F649-4249-97CB-44A0B80086A6}
static const GUID IID_IMaterial = 
{ 0x10200473, 0xf649, 0x4249, { 0x97, 0xcb, 0x44, 0xa0, 0xb8, 0x0, 0x86, 0xa6 } };

interface __declspec(uuid("{10200473-F649-4249-97CB-44A0B80086A6}")) IMaterial: public IDPEObject
{
	// if texture_level less than 0 methods are called for all texture layers.
	virtual HRESULT CALLBACK SetTexture(int texture_level, const char *name) = 0;
	virtual HRESULT CALLBACK GetTexture(int texture_level, OUT ITexture *&pTexture) = 0;
	virtual HRESULT CALLBACK ReleaseTexture(int texture_level) = 0;

	/// With alpha
	virtual HRESULT CALLBACK SetAmbient(const Vector &ambient) = 0;
	virtual HRESULT CALLBACK SetDiffuse(const Vector &diffuse) = 0;
	virtual HRESULT CALLBACK SetSpecular(const Vector &specular) = 0;
	virtual HRESULT CALLBACK SetShininess(float shininess) = 0;		

	virtual HRESULT CALLBACK SetAnisotropy(bool enable) = 0;
	virtual HRESULT CALLBACK SetAlphaTest(bool enable) = 0;

	virtual HRESULT CALLBACK SetNormalTechnique(NORMAL_TECHNIQUE normal_technique) = 0;
	virtual HRESULT CALLBACK SetParallaxTechnique(PARALLAX_TECHNIQUE parallax_technique) = 0;	
	virtual HRESULT CALLBACK SetHeightDeltaScale(float height_delta_scale) = 0;
	virtual HRESULT CALLBACK SetHeightScale(float max_height) = 0;

	virtual HRESULT CALLBACK SetLightingModel(LIGHTING_MODEL lighting_model) = 0;
	virtual HRESULT CALLBACK SetShadowReceiving(bool receive_shadows) = 0;

	virtual HRESULT CALLBACK GetRendererMaterialInterface(OUT IUnknown *&pStandartMaterial) = 0;
};

//-----------------------------------------------------------------------
//IFont//
//-----------------------------------------------------------------------
// {3123141D-BC2B-48fa-83BE-0479097FF070}
static const GUID IID_IFont = 
{ 0x3123141d, 0xbc2b, 0x48fa, { 0x83, 0xbe, 0x4, 0x79, 0x9, 0x7f, 0xf0, 0x70 } };

interface __declspec(uuid("{3123141D-BC2B-48fa-83BE-0479097FF070}")) IFont: public IDPEObject
{
	virtual HRESULT CALLBACK Draw(const char *string, int x, int y, ColorRGB &color = DPE_WHITE, unsigned char alpha = 255, float fScale = 1.f, float angle = 0.f, Point2D_Int &rotCenter = Point2D_Int(0,0)) = 0;

	virtual HRESULT CALLBACK GetTexture(OUT ITexture *&pTexture) = 0;
	virtual HRESULT CALLBACK GetStringWidth(const char* string, OUT unsigned int &width) = 0;
	virtual HRESULT CALLBACK GetStringHeight(const char* string, OUT unsigned int &height) = 0;
	virtual HRESULT CALLBACK GetStringSize(const char* string, OUT unsigned int &width, OUT unsigned int &height) = 0;

	virtual HRESULT CALLBACK GetRendererFontInterface(OUT IUnknown *&pFont) = 0;
};

//-----------------------------------------------------------------------
//IVector//
//-----------------------------------------------------------------------
// {B4AF6936-BAB4-4e95-9E72-94704FC4501E}
static const GUID IID_IVector = 
{ 0xb4af6936, 0xbab4, 0x4e95, { 0x9e, 0x72, 0x94, 0x70, 0x4f, 0xc4, 0x50, 0x1e } };

interface __declspec(uuid("{B4AF6936-BAB4-4e95-9E72-94704FC4501E}")) IVector: public IDPEObject
{
	virtual HRESULT CALLBACK GetAABB(OUT TAABB &aabb) = 0;
	virtual HRESULT CALLBACK GetRendererVectorInterface(OUT IUnknown *&pVectorImage) = 0;
};

//-----------------------------------------------------------------------
//IEmitter//
//-----------------------------------------------------------------------
// {76FAFB40-15B9-4e35-9947-DAC3C537971D}
static const GUID IID_IEmitter = 
{ 0x76fafb40, 0x15b9, 0x4e35, { 0x99, 0x47, 0xda, 0xc3, 0xc5, 0x37, 0x97, 0x1d } };

interface __declspec(uuid("{76FAFB40-15B9-4e35-9947-DAC3C537971D}")) IEmitter: public IDPEObject
{
};

//-----------------------------------------------------------------------
//IResourceManager//
//-----------------------------------------------------------------------
// {86E27D52-8E89-4a0d-A18B-CBBA0F8A7113}
static const GUID IID_IResourceManager = 
{ 0x86e27d52, 0x8e89, 0x4a0d, { 0xa1, 0x8b, 0xcb, 0xba, 0xf, 0x8a, 0x71, 0x13 } };

interface __declspec(uuid("{86E27D52-8E89-4a0d-A18B-CBBA0F8A7113}")) IResourceManager: public IDPEObject
{
	///Load resource from file.
	///@note Descripters is the way resources to be created. The queue of descripters is:
	///TTextureDesc, TMaterialDesc, TMeshDesc, TFontDesc, TVectorDesc, TEmitterDesc.
	///@param file the file name.
	///@param name the resource name. Depricated.
	///@param count the size of descs buffer.
	///@param desc the descs buffer.
	virtual HRESULT CALLBACK Load(const char *file, const char *name, int count = 0, TBaseResourceDesc *desc = NULL) = 0;

	virtual HRESULT CALLBACK CreateMaterial(const char *name, OUT IMaterial *&pMaterial) = 0;
	virtual HRESULT CALLBACK CreateVector(const char *name, size_t primitive_count, const IRenderer::VECTOR_PRIMITIVE primitives[], OUT IVector *&pVector) = 0;
	
	virtual HRESULT CALLBACK GetTexture(const char *name, OUT ITexture *&pTexture) = 0;
	virtual HRESULT CALLBACK GetMaterial(const char *name, OUT IMaterial *&pMaterial) = 0;
	virtual HRESULT CALLBACK GetMesh(const char *name, OUT IMesh *&pMesh) = 0;
	virtual HRESULT CALLBACK GetFont(const char *name, OUT IFont *&pFont) = 0;
	virtual HRESULT CALLBACK GetVector(const char *name, OUT IVector *&pVector) = 0;
	virtual HRESULT CALLBACK GetEmitter(const char *name, OUT IEmitter *&pEmitter) = 0;

	///Free resource by type and name.
	///@note Call Release after to delete object complete.
	///Default resources can not be deleted manually.
	virtual HRESULT CALLBACK Free(DPE_OBJECT_TYPE ResType, const char *name) = 0;

	///Free all resources by type except default.
	///@note Call Release after to delete object completely.
	virtual HRESULT CALLBACK FreeAll(DPE_OBJECT_TYPE ResType) = 0;

	///Free and delete resources with one reference.
	virtual HRESULT CALLBACK DropAll() = 0;	
};

}
#endif
