#include "EventSystem.h"
#include "utils.h"

/*
	SEvent
*/

SEvent::SEvent(const char* name)
{
	_name = new char[strlen(name) + 1];
	strcpy(_name, name);
	_hash = Crc32(name);
}
SEvent::~SEvent()
{	
	if (_name) delete[] _name;
}
void SEvent::AddEventListener(IDelegate1<void,IEvent*> *Listener)
{
	_listeners+= Listener;
}
void SEvent::RemoveEventListener(IDelegate1<void,IEvent*> *Listener)
{
	_listeners-= Listener;
}
HRESULT SEvent::SendEvent(IEvent *Event)
{
	_listeners(Event);
	return S_OK;
}
const char* SEvent::Name()
{
	return _name;
}
ULONG SEvent::Hash()
{
	return _hash;
}

/*
	CEventSystem
*/

CEventSystem* CEventSystem::pInstance = NULL;

CEventSystem::CEventSystem():ref_count(1)
{
	InitCrc32();
}
CEventSystem::~CEventSystem()
{	
	for(EventList::iterator iter = _events.begin(); iter != _events.end(); iter++)
		if(NULL != (*iter)) delete (*iter);
	
	_events.clear();
}

void CEventSystem::AddEventListener(const char* EventName, IDelegate1<void,IEvent*> *Listener)
{
	SEvent *pEvent = new SEvent(EventName);
	for(EventList::iterator iter = _events.begin(); iter != _events.end(); iter++)
		if((*iter)->Hash() == pEvent->Hash())
		{
			(*iter)->AddEventListener(Listener);
			if(pEvent) delete pEvent;
			return;
		}

	pEvent->AddEventListener(Listener);
	_events.push_back(pEvent);
}
void CEventSystem::RemoveEventListener(const char* EventName, IDelegate1<void,IEvent*> *Listener)
{
	SEvent *pEvent = new SEvent(EventName);
	for(EventList::iterator iter = _events.begin(); iter != _events.end(); iter++)
		if((*iter)->Hash() == pEvent->Hash())
			(*iter)->RemoveEventListener(Listener);

	if(pEvent) delete pEvent;
}

HRESULT STDMETHODCALLTYPE CEventSystem::SetMainEventSystem(IEventSystem *MainSystem)
{
	if (!MainSystem) return E_FAIL;
	_mainEventSystem = MainSystem;
	return S_OK;
}
HRESULT STDMETHODCALLTYPE CEventSystem::DispatchEvent(IEvent *Event)
{
	ULONG hash;
	char* name;
	Event->Hash(&hash);
	Event->Name(&name);
	for(EventList::iterator iter = _events.begin(); iter != _events.end(); iter++)
		if((*iter)->Hash() == hash)
		{
			(*iter)->SendEvent(Event);
			return S_OK;
		}

	return E_FAIL;
}
HRESULT STDMETHODCALLTYPE CEventSystem::SendEvent(IEvent *Event)
{
	return CEventSystem::Instance()->_mainEventSystem->DispatchEvent(Event);
}

HRESULT STDMETHODCALLTYPE CEventSystem::QueryInterface(REFIID riid, void **ppvObject)
{
	*ppvObject = NULL;

	if(::memcmp(&riid,&IID_IUnknown,sizeof(GUID)) == 0) 
		*ppvObject = static_cast<IUnknown *>(this);

	else if(::memcmp(&riid,&IID_IEventSystem,sizeof(GUID)) == 0)
		*ppvObject = static_cast<IEventSystem*>(this);
	else return E_NOINTERFACE;

	AddRef();
	return S_OK;		
}

ULONG STDMETHODCALLTYPE CEventSystem::AddRef(void)
{
	ref_count++;
	return ref_count;
}
        
ULONG STDMETHODCALLTYPE CEventSystem::Release(void)
{
	ULONG r = --ref_count;
	if (r <= 0)
	{
    	ref_count = 0;
		r = 0;
		//delete this;
	}
	return r;
}

/*
	CEvent
*/

CEvent::CEvent(const char* name)
{
	
	_name = new char[strlen(name) + 1];
	strcpy(_name, name);
	_hash = Crc32(name);
	
	ref_count = 1;
}
CEvent::~CEvent()
{
	if (_name) delete[] _name;
}

HRESULT STDMETHODCALLTYPE CEvent::Send()
{
	#ifdef CORE
	return CMainEventSystem::Instance()->SendEvent(this);
	#else
	return CEventSystem::Instance()->SendEvent(this);
	#endif
}

HRESULT STDMETHODCALLTYPE CEvent::Hash(ULONG *hash)
{
	*hash = _hash;
	return S_OK;
}
HRESULT STDMETHODCALLTYPE CEvent::Name(char** name)
{
	*name = _name;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CEvent::QueryInterface(REFIID riid, void **ppvObject)
{
	*ppvObject = NULL;

	if(::memcmp(&riid,&IID_IUnknown,sizeof(GUID)) == 0) 
		*ppvObject = static_cast<IUnknown *>(this);

	else if(::memcmp(&riid,&IID_IEvent,sizeof(GUID)) == 0)
		*ppvObject = static_cast<IEvent*>(this);
	else return E_NOINTERFACE;

	AddRef();
	return S_OK;		
}

ULONG STDMETHODCALLTYPE CEvent::AddRef(void)
{
	ref_count++;
	return ref_count;
}
        
ULONG STDMETHODCALLTYPE CEvent::Release(void)
{
	ULONG r = --ref_count;
	if (r <= 0)
	{
		delete this;
	}
	return r;
}

#ifdef CORE

/*
	CMainEventSystem
*/

CMainEventSystem* CMainEventSystem::pInstance = NULL;

CMainEventSystem::CMainEventSystem():ref_count(1)
{
	InitCrc32();
}
CMainEventSystem::~CMainEventSystem()
{
	for(EventList::iterator iter = _events.begin(); iter != _events.end(); iter++)
		if(NULL != (*iter)) delete (*iter);

	/* This failed, subevent systems must be freed in plugins 
	for(EventSystemList::iterator iter = _eventSubSystems.begin(); iter != _eventSubSystems.end(); iter++)
		if(NULL != (*iter)) delete (*iter);
	*/

	_eventSubSystems.clear();
	_events.clear();
}

void CMainEventSystem::AddEventListener(const char* EventName, IDelegate1<void,IEvent*> *Listener)
{
	SEvent *pEvent = new SEvent(EventName);
	for(EventList::iterator iter = _events.begin(); iter != _events.end(); iter++)
		if( (*iter)->Hash() == pEvent->Hash() )
		{
			(*iter)->AddEventListener(Listener);
			if(pEvent) delete pEvent;
			return;
		}

	pEvent->AddEventListener(Listener);
	_events.push_back(pEvent);
}
void CMainEventSystem::RemoveEventListener(const char* EventName, IDelegate1<void,IEvent*> *Listener)
{
	SEvent *pEvent = new SEvent(EventName);
	for(EventList::iterator iter = _events.begin(); iter != _events.end(); iter++)
		if((*iter)->Hash() == pEvent->Hash())
			(*iter)->RemoveEventListener(Listener);

	if(pEvent) delete pEvent;
}

HRESULT CMainEventSystem::AddEventSubSystem(IEventSystem *SubSystem)
{
	if (!SubSystem) return E_FAIL;

	for(EventSystemList::iterator iter = _eventSubSystems.begin(); iter != _eventSubSystems.end(); iter++)
		if((*iter) == SubSystem) return E_FAIL;

	SubSystem->SetMainEventSystem(this);
	_eventSubSystems.push_back(SubSystem);
	return S_OK;
}
HRESULT CMainEventSystem::RemoveEventSubSystem(IEventSystem *SubSystem)
{
	if (!SubSystem) return E_FAIL;

	for(EventSystemList::iterator iter = _eventSubSystems.begin(); iter != _eventSubSystems.end(); iter++)
		if((*iter) == SubSystem)
		{
			if((*iter) != NULL) delete (*iter);
			_eventSubSystems.erase(iter);
			return S_OK;
		}
	return E_FAIL;
}

HRESULT CMainEventSystem::SetMainEventSystem(IEventSystem *MainSystem)
{
	return S_OK;
}
HRESULT CMainEventSystem::DispatchEvent(IEvent *Event)
{
	return SendEvent(Event);
}
HRESULT CMainEventSystem::SendEvent(IEvent *Event)
{
	ULONG hash;
	char* name;
	Event->Hash(&hash);
	Event->Name(&name);

	for(EventList::iterator iter = _events.begin(); iter != _events.end(); iter++)
		if((*iter)->Hash() == hash)
			(*iter)->SendEvent(Event);
	
	for(EventSystemList::iterator iter = _eventSubSystems.begin(); iter != _eventSubSystems.end(); iter++)
		(*iter)->DispatchEvent(Event);

	return S_OK;
}

HRESULT CMainEventSystem::QueryInterface(REFIID riid, void **ppvObject)
{
	*ppvObject = NULL;

	if(::memcmp(&riid,&IID_IUnknown,sizeof(GUID)) == 0) 
		*ppvObject = static_cast<IUnknown *>(this);

	else if(::memcmp(&riid,&IID_IEventSystem,sizeof(GUID)) == 0)
		*ppvObject = static_cast<IEventSystem*>(this);
	else return E_NOINTERFACE;

	AddRef();
	return S_OK;		
}

ULONG CMainEventSystem::AddRef(void)
{
	ref_count++;
	return ref_count;
}
        
ULONG CMainEventSystem::Release(void)
{
	ULONG r = --ref_count;
	if (r <= 0)
	{		
		delete this;		
		return r;
	}
	return r;
}
#endif