/*-----------------------------------------------------------------------
This source file is a part of DPEngine.

Created:		16.06.2008
LastModified:	16.06.2008

Revision history:
	* Created by Mike
	* Revised by

Disctription:
-----------------------------------------------------------------------*/
#include "aabb.h"

namespace DPE_Scenes
{

TAABB::TAABB()
{
	Max = Min = Vector(0.f, 0.f, 0.f, 1.f);
}
TAABB::TAABB(const Vector &vMax, const Vector &vMin)
{
	Max = vMax;
	Min = vMin;
}

int TAABB::Classify(const Vector &vPoint) const
{
	if (vPoint.X < Min.X && vPoint.X > Max.X &&
		vPoint.Y < Min.Y && vPoint.Y > Max.Y &&
		vPoint.Z < Min.Z && vPoint.Z > Max.Z) return -1;

	return INTERSECTS;
}
int TAABB::Classify(const Vector &vA, const Vector &vB) const
{
	return -1;
}
int TAABB::Classify(const TPlane &plane) const
{
	Vector nearPoint(Max);
	Vector farPoint(Min);

	if (plane.Normal.X > 0.f)
	{
		nearPoint.X = Min.X;
		farPoint.X = Max.X;
	}

	if (plane.Normal.Y > 0.f)
	{
		nearPoint.Y = Min.Y;
		farPoint.Y = Max.Y;
	}

	if (plane.Normal.Z > 0.f)
	{
		nearPoint.Z = Min.Z;
		farPoint.Z = Max.Z;
	}

	if (plane.Normal.dot3(nearPoint) + plane.Distance > 0.f)
		return BEYOND;

	if (plane.Normal.dot3(farPoint) + plane.Distance > 0.f)
		return INTERSECTS;

	return BEHIND;
}
int TAABB::Classify(const TAABB &aabb) const
{
	if (Min.X <= aabb.Max.X &&
		Min.Y <= aabb.Max.Y &&
		Min.Z <= aabb.Max.Z &&

		Max.X >= aabb.Min.X &&
		Max.Y >= aabb.Min.Y &&
		Max.Z >= aabb.Min.Z) return INTERSECTS;
	
	return -1;
}
Vector TAABB::GetCenter() const
{
	return (Min + Max)/2;
}
float TAABB::GetDiagonal() const
{
	return (Max - Min).norm3();
}
void TAABB::GetEdges(OUT Vector (&edges)[8]) const
{
	/*
		   Y
		   |
		   |
          /---------/|
         / |       / |
        /  |      /  |
        |------- Max |
        | Min - - - -|- - ---- X
        | /       |  /
        |/        | /
        /----------/ 
	   /
	   Z
	*/

	edges[0] = Min;
	edges[1] = Vector(Max.X, Min.Y, Min.Z);
	edges[2] = Vector(Max.X, Min.Y, Max.Z);
	edges[3] = Vector(Min.X, Min.Y, Max.Z);

	edges[4] = Vector(Min.X, Max.Y, Min.Z);
	edges[5] = Vector(Max.X, Max.Y, Min.Z);
	edges[6] = Max;
	edges[7] = Vector(Min.X, Max.Y, Max.Z);	
}
void TAABB::AddInternalPoint(const Vector &vPoint)
{
	if (vPoint.X > Max.X) Max.X = vPoint.X;
	if (vPoint.Y > Max.Y) Max.Y = vPoint.Y;
	if (vPoint.Z > Max.Z) Max.Z = vPoint.Z;

	if (vPoint.X < Min.X) Min.X = vPoint.X;
	if (vPoint.Y < Min.Y) Min.Y = vPoint.Y;
	if (vPoint.Z < Min.Z) Min.Z = vPoint.Z;
}
TAABB &TAABB::operator += (const TAABB &other)
{
	Vector edges[8];
	other.GetEdges(edges);
	for (int i = 0; i < 8; i++)
		this->AddInternalPoint(edges[i]);
	return *this;
}
TAABB &TAABB::operator *= (const Matrix &m)
{
	TAABB aabb(*this);
	aabb.Max*= m;
	aabb.Min*= m;

	this->Max.X = (aabb.Max.X>aabb.Min.X)?aabb.Max.X:aabb.Min.X;//max(aabb.Max.X, aabb.Min.X);
	this->Max.Y = (aabb.Max.Y>aabb.Min.Y)?aabb.Max.Y:aabb.Min.Y;//max(aabb.Max.Y, aabb.Min.Y);
	this->Max.Z = (aabb.Max.Z>aabb.Min.X)?aabb.Max.Z:aabb.Min.Z;//max(aabb.Max.Z, aabb.Min.Z);
	this->Min.X = (aabb.Max.X<aabb.Min.X)?aabb.Max.X:aabb.Min.X;//min(aabb.Max.X, aabb.Min.X);
	this->Min.Y = (aabb.Max.Y<aabb.Min.Y)?aabb.Max.Y:aabb.Min.Y;//min(aabb.Max.Y, aabb.Min.Y);
	this->Min.Z = (aabb.Max.Z<aabb.Min.Z)?aabb.Max.Z:aabb.Min.Z;//min(aabb.Max.Z, aabb.Min.Z);
	return *this;
}
TAABB operator + (const TAABB &first, const TAABB &second)
{
	return TAABB(first)+= second;
}
TAABB operator * (const TAABB &aabb, const Matrix &m)
{
	return TAABB(aabb)*= m;
}

}