unit e_particles;

interface

uses OpenGL, e_types, e_utils, e_models, e_textures, e_font, e_screen, e_global;

const
  particles_fire            = 1;
  particles_blade           = 2;
  particles_bullet          = 3;
  particles_firework        = 4;
  particles_enemy1          = 5;
  particles_enemy2          = 6;
  particles_enemy3          = 7;
  particles_enemy_boss1     = 8;
  particles_blood           = 9;

type
  TParticle = record
    pos                    : TPoint;
    speed                  : TPoint;
    angle                  : single;
    angle_speed            : single;
    life                   : single;
    start_life             : single;
    size                   : single;
    color                  : TColor;
  end;

  TParticlesEmitter = class
    objects : array of TParticle;
    pos     : TPoint;
    active  : boolean;
    render_to_texture : boolean;
    color   : TColor;
    texture : integer;
    front   : boolean;
    loop    : boolean;
    multiply : boolean;
    parent  : TCreature;
    size    : single;
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   virtual;
    procedure   ResetParticle(i : integer);                    virtual;
    procedure   UpdateParticle(i : integer; count : single);   virtual;
    procedure   Update(count : single);
    procedure   Render;                                        virtual;
  end;

  TParticlesFire = class(TParticlesEmitter)
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   ResetParticle(i : integer);                    override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesBlade = class(TParticlesEmitter)
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   ResetParticle(i : integer);                    override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesBullet = class(TParticlesEmitter)
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   ResetParticle(i : integer);                    override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesFirework = class(TParticlesEmitter)
    strength : single;
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   ResetParticle(i : integer);                    override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesEnemy1 = class(TParticlesEmitter)
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   ResetParticle(i : integer);                    override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesEnemy2 = class(TParticlesEmitter)
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   ResetParticle(i : integer);                    override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesBlood = class(TParticlesEmitter)
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   ResetParticle(i : integer);                    override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesEngine = class
    objects : array of TParticlesEmitter;
  public
    constructor Create;
    function    AddEmitter(m : word; p : TPoint; n : integer; c : TColor) : TParticlesEmitter;
    procedure   DeleteEmitter(i : integer);
    procedure   Update(count : single);
    procedure   RenderBack(blood_only : boolean);
    procedure   RenderFront;
  end;

var
  particles             : TParticlesEngine;
  particles_active      : boolean = true;
  particles_in_frame    : integer = 0;
  particles_updates     : integer = 0;

implementation

uses e_game, e_camera, e_player;

{-- TParticlesEmitter --}

constructor TParticlesEmitter.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create;

pos     := p;
color   := c;
active  := true;
render_to_texture := false;
size    := 1;
end;

procedure TParticlesEmitter.ResetParticle(i : integer);
begin
end;

procedure TParticlesEmitter.UpdateParticle(i : integer; count : single);
begin
end;

procedure TParticlesEmitter.Update(count : single);
var
i, j : integer;
begin
j := 0;
for i := 0 to high(objects) do
  with objects[i] do
    if life > 0 then
      begin
        life := life - count;
        UpdateParticle(i, count);
        inc(j);
      end
    else
      if loop then
        ResetParticle(i);
        
if (not loop) and (j <= 0) then active := false;
end;

procedure TParticlesEmitter.Render;
var
i : integer;
s : single;
begin
SetTexture(texture);

if multiply then
  glBlendFunc(GL_SRC_ALPHA, GL_ONE)
else
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
for i := 0 to high(objects) do
  with objects[i] do
    if life > 0 then
      if CircleVisible(pos, size) then
      begin
        inc(particles_in_frame);
        with color do glColor4f(r, g, b, a);
        glPushMatrix;
        with pos do glTranslatef(x, y, z);
        if parent <> nil then
          glScalef(size * parent.side, size, size)
        else
          glScalef(size, size, size);
        glRotatef(angle, 0, 0, 1);
        glCallList(model_quad);
        glPopMatrix;
      end;

if multiply then glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

end;

{-- TParticlesEmitter --}

{-- TParticlesFire --}

constructor TParticlesFire.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

texture := texture_circle;
front := false;
loop := true;
multiply := false;
SetLength(objects, n);
for i := 0 to high(objects) do
  begin
    ResetParticle(i);
    objects[i].start_life := i / high(objects) * 800;
    objects[i].life := objects[i].start_life;
  end;
end;

procedure TParticlesFire.ResetParticle(i : integer);
begin
objects[i].pos := pos;
objects[i].color := color;
  with objects[i] do
    begin
      pos.x := pos.x;
      pos.y := pos.y;
      angle := game.player.angle;
      size  := 0;

      start_life  := 800;
      life        := start_life;
      speed       := GetPoint(0, 0, 0);
    end;
end;

procedure TParticlesFire.UpdateParticle(i : integer; count : single);
var
s : single;
p : TPoint;
begin
p := pos;
s := 1.5;
if multiply then
  s := s * 0.75;

with objects[i] do
  begin
    //Aspire(pos.x, p.x, count * 0.001);
    //Aspire(pos.y, p.y, count * 0.001);
    
    if life > start_life - 200 then
      size := size + s * abs(1 - size) * count * 0.018
    else
      size := size - s * (size + 1) * count * 0.002;
      
    if size < 0 then size := 0;
  end;
end;

{-- TParticlesFire --}

{-- TParticlesBlade --}

constructor TParticlesBlade.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

texture := texture_blade;
front := false;
loop := true;
multiply := false;
SetLength(objects, n);
for i := 0 to high(objects) do
  begin
    ResetParticle(i);
    objects[i].start_life := i / high(objects) * 150;
    objects[i].life := objects[i].start_life;
  end;
end;

procedure TParticlesBlade.ResetParticle(i : integer);
begin
objects[i].pos := pos;
objects[i].color := color;

  with objects[i] do
    begin
      angle := -360;
      size  := 0;

      start_life  := 150;
      life        := start_life;
      speed       := GetPoint(0, 0, 0);
    end;
end;

procedure TParticlesBlade.UpdateParticle(i : integer; count : single);
const
FULL_PATH = 3 * pi;
var
p : TPoint;
s : single;
begin
p := pos;
if parent <> nil then
  objects[i].angle := parent.angle * 180 / pi;

objects[i].pos := pos;

with objects[i] do
  begin
    s := 1.5;
    if multiply then
      s := 2;

    size := s * life / start_life;
    color.a := sqr(life / start_life);
  end;
end;

{-- TParticlesBlade --}

{-- TParticlesBullet --}

constructor TParticlesBullet.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

texture := texture_circle;
front := false;
loop := true;
multiply := false;
SetLength(objects, n);
for i := 0 to high(objects) do
  begin
    ResetParticle(i);
    objects[i].start_life := i / high(objects) * 200;
    objects[i].life := objects[i].start_life;
  end;
end;

procedure TParticlesBullet.ResetParticle(i : integer);
begin
objects[i].pos := pos;
objects[i].color := color;
  with objects[i] do
    begin
      angle := 0;
      size  := 0;

      start_life  := 200;
      life        := start_life;
      speed       := GetPoint(0, 0, 0);
    end;
end;

procedure TParticlesBullet.UpdateParticle(i : integer; count : single);
var
s : single;
begin
with objects[i] do
  begin
    s := 1;
    if multiply then
      s := 0.7;

    size := s * life / start_life;
    color.a := life / start_life;
  end;
end;

{-- TParticlesBullet --}

{-- TParticlesFirework --}

constructor TParticlesFirework.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

strength := 0.5;
texture := texture_circle;
front := false;
loop := true;
multiply := false;
SetLength(objects, n);
for i := 0 to high(objects) do
  begin
    ResetParticle(i);
    objects[i].start_life := i / high(objects) * 220;
    objects[i].life := objects[i].start_life;
  end;
end;

procedure TParticlesFirework.ResetParticle(i : integer);
begin
objects[i].pos := pos;
objects[i].color := color;
  with objects[i] do
    begin
      size  := 0;

      start_life  := 220;
      life        := start_life;
      angle := random * 2 * pi;
      speed       := GetPoint(cos(angle) * strength, sin(angle) * strength, 0);
      angle := random * 360;
    end;
end;

procedure TParticlesFirework.UpdateParticle(i : integer; count : single);
var
p : TPoint;
s : single;
begin
p := pos;
with objects[i] do
  begin
    s := 1;
    if multiply then
      s := 0.7;

    pos.x := pos.x + speed.x * count * 0.001;
    pos.y := pos.y + speed.y * count * 0.001;

    size := 0.7 * s * life / start_life;
    color.a := life / start_life;
  end;
end;

{-- TParticlesFirework --}

{-- TParticlesEnemy1 --}

constructor TParticlesEnemy1.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

texture := texture_circle;
front := false;
loop := true;
multiply := false;
SetLength(objects, n);
for i := 0 to high(objects) do
  begin
    ResetParticle(i);
    objects[i].start_life := i / high(objects) * 400;
    objects[i].life := objects[i].start_life;
  end;
end;

procedure TParticlesEnemy1.ResetParticle(i : integer);
begin
objects[i].pos := pos;
objects[i].color := color;
objects[i].size  := size;
if parent <> nil then
  objects[i].angle := parent.angle;
  
  with objects[i] do
    begin
      pos.x := pos.x;
      pos.y := pos.y;         

      color.a := 1;
      start_life  := 400;
      life        := start_life;
      speed       := GetPoint(0, 0, 0);
    end;
end;

procedure TParticlesEnemy1.UpdateParticle(i : integer; count : single);
var
p : TPoint;
s : single;
begin
p := pos;

with objects[i] do
  begin
    Aspire(size, 0, count * 0.005);
    SqrAspire(color.a, size - 1, count * 0.02);
  end;
end;

{-- TParticlesEnemy1 --}

{-- TParticlesEnemy2Front --}

constructor TParticlesEnemy2.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

texture := texture_enemy2;
front := false;
loop := true;
multiply := false;
SetLength(objects, n);
size := 3;
for i := 0 to high(objects) do
  begin
    ResetParticle(i);
    objects[i].start_life := i / high(objects) * 400;
    objects[i].life := objects[i].start_life;
  end;
end;

procedure TParticlesEnemy2.ResetParticle(i : integer);
begin
objects[i].pos := pos;
objects[i].color := color;
if parent <> nil then
  objects[i].angle := parent.angle;
  
  with objects[i] do
    begin
      size  := 0;

      start_life  := 400;
      life        := start_life;
      speed       := GetPoint(GetRandom, 3 + random * 3, 0);
    end;
end;

procedure TParticlesEnemy2.UpdateParticle(i : integer; count : single);
var
p : TPoint;
s : single;
begin
p := pos;
s := size;
with objects[i] do
  begin
    pos.x := pos.x + speed.x * count * 0.001;
    pos.y := pos.y + speed.y * count * 0.001;

    size := s * life / start_life;
    color.a := life / start_life;
  end;
end;

{-- TParticlesEnemy2Front --}

{-- TParticlesBlood --}

constructor TParticlesBlood.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

texture := texture_circle;
front := true;
loop := false;
multiply := true;
SetLength(objects, n);
for i := 0 to high(objects) do
  ResetParticle(i);
end;

procedure TParticlesBlood.ResetParticle(i : integer);
begin
objects[i].pos := pos;
objects[i].color := color;
  with objects[i] do
    begin
      pos.x := pos.x;
      pos.y := pos.y;
      angle := random * 360;
      size  := 1 + random;

      start_life  := 300 + random * 300;
      life        := start_life;
      speed       := GetPoint(cos(angle) * size * 2, sin(angle) * size * 2, 0);
    end;
end;

procedure TParticlesBlood.UpdateParticle(i : integer; count : single);
var
s : single;
p : TPoint;
begin
with objects[i] do
  begin
    pos.x := pos.x + speed.x * count * 0.001;
    pos.y := pos.y + speed.y * count * 0.001;

    size := life / start_life;
  end;
end;

{-- TParticlesBlood --}


{-- TParticlesEngine --}

constructor TParticlesEngine.Create;
var
i : integer;
x, z : single;
begin
SetLength(objects, 0);
end;

function TParticlesEngine.AddEmitter(m : word; p : TPoint; n : integer; c : TColor) : TParticlesEmitter;
var
i, max : integer;
begin
result := nil;

i   := 0;
max := high(objects);
while (i <= max) and (objects[i] <> nil) do inc(i);
if i > max then SetLength(objects, max + 4);

if not render_particles_high then
  if m > 2 then
    m := m div 2;

case m of
  particles_fire             : objects[i] := TParticlesFire.Create(p, n, c);
  particles_blade            : objects[i] := TParticlesBlade.Create(p, n, c);
  particles_bullet           : objects[i] := TParticlesBullet.Create(p, n, c);
  particles_firework         : objects[i] := TParticlesFirework.Create(p, n, c);
  particles_enemy1           : objects[i] := TParticlesEnemy1.Create(p, n, c);
  particles_enemy2           : objects[i] := TParticlesEnemy2.Create(p, n, c);
  particles_blood            : objects[i] := TParticlesBlood.Create(p, n, c);
end;

{ 

  particles_enemy3          = 7;
  particles_enemy_boss1     = 8;}

result := objects[i];
end;

procedure TParticlesEngine.DeleteEmitter(i : integer);
begin
objects[i].Free;
objects[i] := nil;
end;

procedure TParticlesEngine.Update(count : single);
var
i : integer;
begin
if not particles_active then exit;

particles_updates := 0;
for i := 0 to high(objects) do
  if objects[i] <> nil then
    if objects[i].active then
      begin
        objects[i].Update(count);
        inc(particles_updates);
      end
    else
      DeleteEmitter(i);
end;

procedure TParticlesEngine.RenderBack(blood_only : boolean);
var
i : integer;
begin
particles_in_frame := 0;

if not particles_active then exit;

for i := 0 to high(objects) do
  if objects[i] <> nil then
    if objects[i].active and (not objects[i].front) then
      //if (not blood_only) or (blood_only and (objects[i] is TParticlesBlood)) then
      //if (blood_only and ((objects[i] is TParticlesBlood) or (objects[i] is TParticlesExplosionSoot))) then
      if (not blood_only) or (objects[i].render_to_texture and blood_only) then
        objects[i].Render;
end;

procedure TParticlesEngine.RenderFront;
var
i : integer;
begin
if not particles_active then exit;

for i := 0 to high(objects) do
  if objects[i] <> nil then
    if objects[i].active and objects[i].front then
      objects[i].Render;
end;

{-- TParticlesEngine --}

end.
