
#include <math.h>
#include <float.h>

#include "Globals.h"
#include "Animators.h"

/* --- Animator class --- */

Animator::Animator()
{
	SetState(ANIM_STATE_READY);

	TimeStarted = 0.0f;
	TimeLoop = 0.0f;
	TimeFull = 0.0f;

	DeltaStart = 0.0f;

	Delay0 = 0.0f;
	Delay1 = 0.0f;

	Period = 0.0f;
	Interval = 1.0f;

	PeriodVariation = 0.0f;

	LoopIndex = 0;
	LoopsCount = 0;

	Delayed = true;
}

void Animator::SetState(AnimatorState s)
{
	State = s;

	Running = (s == ANIM_STATE_RUN);
	Standby = (s == ANIM_STATE_READY);
	Ended = (s == ANIM_STATE_END);
}

void Animator::Start(float Time)
{
	Initialize(Time);

	TimeLoop = Time;
	TimeStarted = Time;
	
	DeltaStart = 0.0f;
	TimeFull = Delay0 + Interval + Delay1;

	if (PeriodVariation > 0.0f)
	{
		float v = random(PeriodVariation);
		Period += v - PeriodVariation / 2.0f;
	}

	SetState(ANIM_STATE_RUN);
}

void Animator::Stop()
{
	Delayed = true;
	DeltaStart = 0.0f;

	SetState(ANIM_STATE_END);
}

void Animator::Update(float Time)
{
	float Delta = Time - TimeLoop;
	DeltaStart = Time - TimeStarted;

	// Check if next loop possible
	bool Loop = ((State == ANIM_STATE_END) && (Delta > Period) && 
		((LoopsCount < LoopIndex) || (LoopsCount == -1)));
	
	if (Loop)
	{
		Start(Time);
		LoopIndex++;
		return;
	}

	// Exit if not running
	if (State != ANIM_STATE_RUN) return;

	// Is play delayed?
	bool delayed = (DeltaStart < Delay0);

	// Infinite run
	if (Interval > 0.0f)
	{
		// Stop if needed
		if (DeltaStart > TimeFull) 
		{
			float TimeEnd = TimeStarted + Interval;
			Calculate(TimeEnd, Interval);

			Stop();
			return;
		}

		// Check right delay and combine with previous
		delayed = (DeltaStart > Delay0 + Interval) || delayed;
	}

	Delayed = delayed;
	if (delayed) return;

	Calculate(Time - Delay0, DeltaStart - Delay0);
}

/* --- Linear animator class --- */

LinearAnimator::LinearAnimator()
{
	Value = 0.0f;
	Value0 = 0.0f;
	Value1 = 1.0f;

	ValueNorm = 0.0f;
	ValueRange = 1.0f;
}

void LinearAnimator::SetBounds(float A, float B)
{
	Value0 = A;
	Value1 = B;

	ValueRange = B - A;
}

void LinearAnimator::Calculate(float Time, float dT)
{
	Value = Value0 + ValueRange * dT / Interval;
	ValueNorm = Value / Value1;
}

void LinearAnimator::Initialize(float Time)
{
	Value = Value0;
	ValueNorm = 0.0f;
}

/* --- Acceleration animator class --- */

AccelerationAnimator::AccelerationAnimator()
{
	Position0[0] = 0.0f;
	Position0[1] = 1.0f;

	Velocity0[0] = 0.0f;
	Velocity0[1] = 0.0f;

	Acceleration0[0] =  0.0f;
	Acceleration0[1] = -1.0f;
}

void AccelerationAnimator::Initialize(float Time)
{
	Position[0] = Position0[0];
	Position[1] = Position0[1];

	Velocity[0] = Velocity0[0];
	Velocity[1] = Velocity0[1];

	Acceleration[0] = Acceleration0[0];
	Acceleration[1] = Acceleration0[1];

	Time0 = Time;
}

void AccelerationAnimator::SetPosition(float X, float Y)
{
	Position0[0] = X;
	Position0[1] = Y;
}

void AccelerationAnimator::SetVelocity(float X, float Y)
{
	Velocity0[0] = X;
	Velocity0[1] = Y;
}

void AccelerationAnimator::SetAcceleration(float X, float Y)
{
	Acceleration0[0] = X;
	Acceleration0[1] = Y;
}

void AccelerationAnimator::UpdateVelocity(float dT0)
{
	// Save previous velocity
	Velocity0[0] = Velocity[0];
	Velocity0[1] = Velocity[1];

	// Calculate new velocity
	CalculateVelocity(Velocity, dT0);
}

void AccelerationAnimator::CalculateVelocity(Vector2f& v, float dT0)
{
	v[0] = Velocity0[0] + Acceleration[0] * dT0;
	v[1] = Velocity0[1] + Acceleration[1] * dT0;
}

void AccelerationAnimator::UpdatePosition(float dT0)
{
	// Save previous position
	Position0[0] = Position[0];
	Position0[1] = Position[1];

	// Calculate new position
	CalculatePosition(Position, dT0);
}

void AccelerationAnimator::CalculatePosition(Vector2f& p, float dT0)
{
	p[0] = Position0[0] + Velocity[0] * dT0 + Acceleration[0] * dT0 * dT0 / 2.0f;
	p[1] = Position0[1] + Velocity[1] * dT0 + Acceleration[1] * dT0 * dT0 / 2.0f;
}

void AccelerationAnimator::Calculate(float Time, float dT)
{
	float dT0 = Time - Time0;

	UpdateVelocity(dT0);
	UpdatePosition(dT0);
}

/* --- Gravity animator class -- */

GravityAnimator::GravityAnimator()
{
	HitIndex = 0;
	HitsCount = 0;

	Throttle = 2.0f;
	Friction = 1.0f;

	Speed[0] = 0.0f;
	Speed[1] = 0.0f;
	
	Ground = 0.0f;
	Landed = false;
	GroundHit = false;
	MoveActive = false;
}

void GravityAnimator::Initialize(float Time)
{
	AccelerationAnimator::Initialize(Time);
	HitIndex = 0;
}

void GravityAnimator::UpdateState(float Time)
{
	float dT0 = Time - Time0;

	GroundHit = false;

	if (Position[1] >= Ground)
	{
		if (Position0[1] < Ground)
		{
			Vector2f ht = { 0.0f, 0.0f };	
			GroundHit = CalculateHitGround(ht);

			if (GroundHit)
			{
				// Counter
				HitIndex++;

				// Position and speed at hit point
				CalculatePosition(Position, ht[0]);
				CalculateVelocity(Velocity, ht[0]); 

				// First drop check
				if (Position[1] > Ground)
					Position[1] = Ground;
	
				if ((HitsCount == -1) || 
					(HitIndex <= HitsCount))
				{
					// Time delta after hit
					float hdt = dT0 - ht[0];
					if (hdt < 0.0f) hdt = 0.0f;

					float TotalSpeed = Velocity[0] + Speed[0];
					Velocity[0] = TotalSpeed * 0.5f;
					Speed[0] = TotalSpeed * 0.5f;

					if (Speed[1] != 0.0f)
						Velocity[1] = Speed[1];
					else
						Velocity[1] /= -Throttle;
				
					// Update 
					UpdatePosition(hdt);
					UpdateVelocity(hdt);
				}
			}
		}

		// Second drop check
		if (Position[1] > Ground)
			Position[1] = Ground;
	}

	// Check whether animator landed
	Landed = (Position[1] == Ground) && 
		(Position0[1] == Ground) && (Velocity[1] > 0.0f);

	// bool HitsAllow = (HitsCount == -1) || (HitIndex < HitsCount);

	if (Landed)
	{
		HitIndex = 0;

		Velocity[1] = 0.0f;
		Speed[1] = 0.0f;

		if (!MoveActive) 
		{
			Velocity[0] *= (1.0f - Friction * dT0);
			Speed[0] = Velocity[0];
		}
	}
	else
		Speed[0] *= (1.0f - Breaking * dT0);
}

bool GravityAnimator::CalculateHitGround(Vector2f& h)
{
	float a = Acceleration0[1] / 2.0f;
	float b = Velocity0[1];
	float c = Position0[1] - Ground;

	if (a == 0.0f)
	{
		if (b == 0.0f)
		{
			return false;
		}
		else
		{
			h[0] = -c / b;
			h[1] = h[0];

			return true;
		}
	}
	else
	{
		float d = b * b - 4.0f * a * c;
		if (d < 0.0f) return false;

		d = sqrt(d);
	
		h[0] = (-b + d) / (2.0f * a);
		h[1] = (-b - d) / (2.0f * a);

		return true;
	}
}


void GravityAnimator::Calculate(float Time, float dT)
{
	AccelerationAnimator::Calculate(Time, dT);
	UpdateState(Time);
	Time0 = Time;
}

void GravityAnimator::GetStartVelocity(float Speed, Vector2f& Target, Vector2f& Velocity1, Vector2f& Velocity2)
{
	float dx = Position0[0] - Target[0];
	float dy = Position0[1] - Target[1];

	float A = (Acceleration0[0] * Acceleration0[0] + 
		Acceleration0[1] * Acceleration0[1]) / 4.0f;

	float B = dx * dx + dy * dy;
	float C = Acceleration0[0] * dx + Acceleration0[1] * dy;

	float sc = Speed * Speed - C;
	float D = sc * sc - 4.0f * A * B;

	if (D >= 0.0f) D = sqrt(D);
	else return;

	float t1 = (sc + D) / (2.0f * A);
	float t2 = (sc - D) / (2.0f * A);

	if (t1 > 0.0f) t1 = sqrt(t1);
	if (t2 > 0.0f) t2 = sqrt(t2);

	if (t1 > 0.0f)
	{
		Velocity1[0] = -dx / t1 - Acceleration0[0] * t1 / 2.0f;
		Velocity1[1] = -dy / t1 - Acceleration0[1] * t1 / 2.0f;
	}

	if (t2 > 0.0f)
	{
		Velocity2[0] = -dx / t2 - Acceleration0[0] * t2 / 2.0f;
		Velocity2[1] = -dy / t2 - Acceleration0[1] * t2 / 2.0f;
	}
}